@@ -479,9 +479,6 @@ namespace dxvk {
       const Rc<DxvkInstance>& Instance,
       const Rc<DxvkAdapter>&  Adapter);
     
-    static DxvkDeviceFeatures GetDeviceFeatures(
-      const Rc<DxvkAdapter>&  Adapter);
-
     DxvkBarrierControlFlags GetOptionsBarrierControlFlags() {
       DxvkBarrierControlFlags barrierControl = 0u;
 
@@ -991,6 +988,8 @@ namespace dxvk {
     DXGIDXVKDevice      m_metaDevice;
     
     DXGIVkSwapChainFactory   m_dxvkFactory;
+
+    D3DDestructionNotifier   m_destructionNotifier;
     
     uint32_t m_frameLatency = DefaultFrameLatency;
 
