@@ -13,19 +13,6 @@ namespace dxvk {
   struct D3D11Options {
     D3D11Options(const Config& config);
 
-    /// Enables speed hack for mapping on deferred contexts
-    ///
-    /// This can substantially speed up some games, but may
-    /// cause issues if the game submits command lists more
-    /// than once.
-    bool dcSingleUseMode = false;
-
-    /// Zero-initialize workgroup memory
-    ///
-    /// Workargound for games that don't initialize
-    /// TGSM in compute shaders before reading it.
-    bool zeroInitWorkgroupMemory = false;
-
     /// Force thread-group shared memory accesses to be volatile
     ///
     /// Workaround for compute shaders that read and
@@ -33,6 +20,13 @@ namespace dxvk {
     /// without explicit synchronization.
     bool forceVolatileTgsmAccess = false;
 
+    /// Force UAV synchronization insided compute shaders
+    ///
+    /// Workaround for compute shaders that access overlapping
+    /// memory regions within a UAV without proper workgroup
+    /// synchroniation. Will have a negative performance impact.
+    bool forceComputeUavBarriers = false;
+
     /// Use relaxed memory barriers
     ///
     /// May improve performance in some games,
@@ -43,7 +37,7 @@ namespace dxvk {
     ///
     /// May improve performance in some games,
     /// but might also cause rendering issues.
-    bool ignoreGraphicsBarriers = false;
+    bool relaxedGraphicsBarriers = false;
 
     /// Maximum tessellation factor.
     ///
@@ -72,10 +66,6 @@ namespace dxvk {
     /// Enable float control bits
     bool floatControls = true;
 
-    /// Back buffer count for the Vulkan swap chain.
-    /// Overrides DXGI_SWAP_CHAIN_DESC::BufferCount.
-    int32_t numBackBuffers = 0;
-
     /// Override maximum frame latency if the app specifies
     /// a higher value. May help with frame timing issues.
     int32_t maxFrameLatency = 0;
@@ -114,8 +104,15 @@ namespace dxvk {
     /// can negatively affect performance.
     bool reproducibleCommandStream = false;
 
+    /// Whether to force a staging buffer for mapped images.
+    /// Some games are broken and ignore row pitch.
+    bool disableDirectImageMapping = false;
+
+    /// Whether to use sincos emulation
+    Tristate sincosEmulation = Tristate::Auto;
+
     /// Shader dump path
     std::string shaderDumpPath;
   };
   
-}
\ No newline at end of file
+}
