@@ -8,60 +8,59 @@ namespace dxvk {
           D3D11Device*        device,
     const D3D11_SAMPLER_DESC& desc)
   : D3D11StateObject<ID3D11SamplerState>(device),
-    m_desc(desc), m_d3d10(this) {
-    DxvkSamplerCreateInfo info;
-    
+    m_desc(desc), m_d3d10(this), m_destructionNotifier(this) {
+    DxvkSamplerKey info = { };
+
     // While D3D11_FILTER is technically an enum, its value bits
     // can be used to decode the filter properties more efficiently.
     const uint32_t filterBits = uint32_t(desc.Filter);
-    info.magFilter      = (filterBits & 0x04) ? VK_FILTER_LINEAR : VK_FILTER_NEAREST;
-    info.minFilter      = (filterBits & 0x10) ? VK_FILTER_LINEAR : VK_FILTER_NEAREST;
-    
+
+    VkFilter minFilter = (filterBits & 0x10) ? VK_FILTER_LINEAR : VK_FILTER_NEAREST;
+    VkFilter magFilter = (filterBits & 0x04) ? VK_FILTER_LINEAR : VK_FILTER_NEAREST;
+
+    info.setFilter(minFilter, magFilter,
+      (filterBits & 0x01) ? VK_SAMPLER_MIPMAP_MODE_LINEAR : VK_SAMPLER_MIPMAP_MODE_NEAREST);
+
+    // Enforce LOD bias specified in the device options
+    float lodBias = desc.MipLODBias;
+
+    if (minFilter == VK_FILTER_LINEAR && magFilter == VK_FILTER_LINEAR) {
+      lodBias += device->GetOptions()->samplerLodBias;
+
+      if (device->GetOptions()->clampNegativeLodBias)
+        lodBias = std::max(lodBias, 0.0f);
+    }
+
+    info.setLodRange(desc.MinLOD, desc.MaxLOD, lodBias);
+
+    // Enforce anisotropy specified in the device options
+    uint32_t anisotropy = (filterBits & 0x40) ? desc.MaxAnisotropy : 0u;
+    int32_t samplerAnisotropyOption = device->GetOptions()->samplerAnisotropy;
+
+    if (samplerAnisotropyOption >= 0 && minFilter == VK_FILTER_LINEAR)
+      anisotropy = samplerAnisotropyOption;
+
+    info.setAniso(anisotropy);
+
     // Set up the remaining properties, which are
     // stored directly in the sampler description
-    info.mipmapMode     = (filterBits & 0x01) ? VK_SAMPLER_MIPMAP_MODE_LINEAR : VK_SAMPLER_MIPMAP_MODE_NEAREST;
-    info.mipmapLodBias  = desc.MipLODBias;
-    info.mipmapLodMin   = desc.MinLOD;
-    info.mipmapLodMax   = desc.MaxLOD;
-    
-    info.useAnisotropy  = (filterBits & 0x40) ? VK_TRUE : VK_FALSE;
-    info.maxAnisotropy  = float(desc.MaxAnisotropy);
-    
-    info.addressModeU   = DecodeAddressMode(desc.AddressU);
-    info.addressModeV   = DecodeAddressMode(desc.AddressV);
-    info.addressModeW   = DecodeAddressMode(desc.AddressW);
-    
-    info.compareToDepth = (filterBits & 0x180) == 0x80 ? VK_TRUE : VK_FALSE;
-    info.compareOp      = DecodeCompareOp(desc.ComparisonFunc);
+    info.setAddressModes(
+      DecodeAddressMode(desc.AddressU),
+      DecodeAddressMode(desc.AddressV),
+      DecodeAddressMode(desc.AddressW));
 
-    info.reductionMode  = DecodeReductionMode(filterBits);
+    info.setDepthCompare((filterBits & 0x180) == 0x80,
+      DecodeCompareOp(desc.ComparisonFunc));
+
+    info.setReduction(DecodeReductionMode(filterBits));
 
     for (uint32_t i = 0; i < 4; i++)
       info.borderColor.float32[i] = desc.BorderColor[i];
-    
-    info.usePixelCoord  = VK_FALSE;  // Not supported in D3D11
-    info.nonSeamless    = VK_FALSE;
-    
-    // Make sure to use a valid anisotropy value
-    if (desc.MaxAnisotropy <  1) info.maxAnisotropy =  1.0f;
-    if (desc.MaxAnisotropy > 16) info.maxAnisotropy = 16.0f;
-    
-    // Enforce LOD bias specified in the device options
-    if (info.minFilter == VK_FILTER_LINEAR && info.magFilter == VK_FILTER_LINEAR)
-      info.mipmapLodBias += device->GetOptions()->samplerLodBias;
-
-    // Enforce anisotropy specified in the device options
-    int32_t samplerAnisotropyOption = device->GetOptions()->samplerAnisotropy;
-
-    if (samplerAnisotropyOption >= 0 && info.minFilter == VK_FILTER_LINEAR) {
-      info.useAnisotropy = samplerAnisotropyOption > 0;
-      info.maxAnisotropy = float(samplerAnisotropyOption);
-    }
 
     m_sampler = device->GetDXVKDevice()->createSampler(info);
   }
-  
-  
+
+
   D3D11SamplerState::~D3D11SamplerState() {
     
   }
@@ -85,7 +84,12 @@ namespace dxvk {
       *ppvObject = ref(&m_d3d10);
       return S_OK;
     }
-    
+
+    if (riid == __uuidof(ID3DDestructionNotifier)) {
+      *ppvObject = ref(&m_destructionNotifier);
+      return S_OK;
+    }
+
     if (logQueryInterfaceError(__uuidof(ID3D11SamplerState), riid)) {
       Logger::warn("D3D11SamplerState::QueryInterface: Unknown interface query");
       Logger::warn(str::format(riid));
