@@ -1143,7 +1143,8 @@ namespace dxvk {
     m_interop (this, &m_texture),
     m_surface (this, &m_texture),
     m_resource(this, pDevice),
-    m_d3d10   (this) {
+    m_d3d10   (this),
+    m_destructionNotifier(this) {
     
   }
   
@@ -1198,6 +1199,11 @@ namespace dxvk {
       return S_OK;
     }
     
+    if (riid == __uuidof(ID3DDestructionNotifier)) {
+      *ppvObject = ref(&m_destructionNotifier);
+      return S_OK;
+    }
+
     if (logQueryInterfaceError(__uuidof(ID3D10Texture1D), riid)) {
       Logger::warn("D3D11Texture1D::QueryInterface: Unknown interface query");
       Logger::warn(str::format(riid));
@@ -1255,7 +1261,8 @@ namespace dxvk {
     m_surface   (this, &m_texture),
     m_resource  (this, pDevice),
     m_d3d10     (this),
-    m_swapChain (nullptr) {
+    m_swapChain (nullptr),
+    m_destructionNotifier(this) {
   }
 
 
@@ -1270,7 +1277,8 @@ namespace dxvk {
     m_surface   (this, &m_texture),
     m_resource  (this, pDevice),
     m_d3d10     (this),
-    m_swapChain (nullptr) {
+    m_swapChain (nullptr),
+    m_destructionNotifier(this) {
     
   }
 
@@ -1286,7 +1294,8 @@ namespace dxvk {
     m_surface   (this, &m_texture),
     m_resource  (this, pDevice),
     m_d3d10     (this),
-    m_swapChain (pSwapChain) {
+    m_swapChain (pSwapChain),
+    m_destructionNotifier(this) {
     
   }
   
@@ -1367,6 +1376,11 @@ namespace dxvk {
       return S_OK;
     }
     
+    if (riid == __uuidof(ID3DDestructionNotifier)) {
+      *ppvObject = ref(&m_destructionNotifier);
+      return S_OK;
+    }
+
     if (logQueryInterfaceError(__uuidof(ID3D10Texture2D), riid)) {
       Logger::warn("D3D11Texture2D::QueryInterface: Unknown interface query");
       Logger::warn(str::format(riid));
@@ -1438,7 +1452,8 @@ namespace dxvk {
     m_texture (this, pDevice, pDesc, p11on12Info, D3D11_RESOURCE_DIMENSION_TEXTURE3D, 0, VK_NULL_HANDLE, nullptr),
     m_interop (this, &m_texture),
     m_resource(this, pDevice),
-    m_d3d10   (this) {
+    m_d3d10   (this),
+    m_destructionNotifier(this) {
     
   }
   
@@ -1485,6 +1500,11 @@ namespace dxvk {
       *ppvObject = ref(&m_interop);
       return S_OK;
     }
+
+    if (riid == __uuidof(ID3DDestructionNotifier)) {
+      *ppvObject = ref(&m_destructionNotifier);
+      return S_OK;
+    }
     
     if (logQueryInterfaceError(__uuidof(ID3D10Texture3D), riid)) {
       Logger::warn("D3D11Texture3D::QueryInterface: Unknown interface query");
