@@ -11,7 +11,8 @@ namespace dxvk {
           ID3D11Resource*                    pResource,
     const D3D11_UNORDERED_ACCESS_VIEW_DESC1* pDesc)
   : D3D11DeviceChild<ID3D11UnorderedAccessView1>(pDevice),
-    m_resource(pResource), m_desc(*pDesc) {
+    m_resource(pResource), m_desc(*pDesc),
+    m_destructionNotifier(this) {
     ResourceAddRefPrivate(m_resource);
 
     D3D11_COMMON_RESOURCE_DESC resourceDesc;
@@ -58,6 +59,7 @@ namespace dxvk {
       
       DxvkImageViewKey viewInfo;
       viewInfo.format = formatInfo.Format;
+      viewInfo.layout = VK_IMAGE_LAYOUT_GENERAL;
       viewInfo.aspects = formatInfo.Aspect;
       viewInfo.usage = VK_IMAGE_USAGE_STORAGE_BIT;
 
@@ -127,6 +129,8 @@ namespace dxvk {
   
   
   D3D11UnorderedAccessView::~D3D11UnorderedAccessView() {
+    m_destructionNotifier.Notify();
+
     ResourceReleasePrivate(m_resource);
     m_resource = nullptr;
 
@@ -150,7 +154,12 @@ namespace dxvk {
       *ppvObject = ref(this);
       return S_OK;
     }
-    
+
+    if (riid == __uuidof(ID3DDestructionNotifier)) {
+      *ppvObject = ref(&m_destructionNotifier);
+      return S_OK;
+    }
+
     if (logQueryInterfaceError(__uuidof(ID3D11UnorderedAccessView), riid)) {
       Logger::warn("D3D11UnorderedAccessView::QueryInterface: Unknown interface query");
       Logger::warn(str::format(riid));
@@ -450,6 +459,7 @@ namespace dxvk {
                 | VK_ACCESS_TRANSFER_READ_BIT
                 | VK_ACCESS_SHADER_WRITE_BIT
                 | VK_ACCESS_SHADER_READ_BIT;
+    info.debugName = "UAV counter";
 
     Rc<DxvkBuffer> buffer = device->createBuffer(info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 
