@@ -19,15 +19,9 @@ IDxvkD3D8Bridge : public IUnknown {
   // D3D8 keeps D3D9 objects contained in a namespace.
   #ifdef DXVK_D3D9_NAMESPACE
     using IDirect3DSurface9 = d3d9::IDirect3DSurface9;
+    using D3DFORMAT = d3d9::D3DFORMAT;
   #endif
 
-  /**
-   * \brief Changes the API name displayed on the HUD
-   *
-   * \param [in] name The new API name
-   */
-  virtual void SetAPIName(const char* name) = 0;
-
   /**
    * \brief Updates a D3D9 surface from a D3D9 buffer
    *
@@ -41,6 +35,13 @@ IDxvkD3D8Bridge : public IUnknown {
       IDirect3DSurface9*        pSrcSurface,
       const RECT*               pSrcRect,
       const POINT*              pDestPoint) = 0;
+
+  /**
+   * \brief Checks if a particular surface format is supported by D3D9
+   *
+   * \param [in] Format D3DFORMAT value to be checked
+   */
+  virtual bool IsSupportedSurfaceFormat(D3DFORMAT Format) = 0;
 };
 
 /**
@@ -49,11 +50,9 @@ IDxvkD3D8Bridge : public IUnknown {
 MIDL_INTERFACE("D3D9D3D8-A407-773E-18E9-CAFEBEEF3000")
 IDxvkD3D8InterfaceBridge : public IUnknown {
   /**
-   * \brief Enables or disables D3D9-specific features and validations
-   *
-   * \param [in] compatMode Compatibility state
+   * \brief Enforces D3D8-specific features and validations
    */
-  virtual void SetD3D8CompatibilityMode(const bool compatMode) = 0;
+  virtual void EnableD3D8CompatibilityMode() = 0;
 
   /**
    * \brief Retrieves the DXVK configuration
@@ -87,14 +86,14 @@ namespace dxvk {
             REFIID  riid,
             void** ppvObject);
 
-    void SetAPIName(const char* name);
-
     HRESULT UpdateTextureFromBuffer(
         IDirect3DSurface9*        pDestSurface,
         IDirect3DSurface9*        pSrcSurface,
         const RECT*               pSrcRect,
         const POINT*              pDestPoint);
 
+    bool IsSupportedSurfaceFormat(D3DFORMAT Format);
+
   private:
 
     D3D9DeviceEx* m_device;
@@ -115,7 +114,7 @@ namespace dxvk {
             REFIID  riid,
             void** ppvObject);
 
-    void SetD3D8CompatibilityMode(const bool compatMode);
+    void EnableD3D8CompatibilityMode();
 
     const Config* GetConfig() const;
 
