@@ -44,16 +44,14 @@
 #define D3DPRESENT_FORCEIMMEDIATE              0x00000100L
 #endif
 
-#ifndef D3DSWAPEFFECT_COPY_VSYNC
-#define D3DSWAPEFFECT_COPY_VSYNC 4
-#endif
-
-// MinGW headers are broken. Who'dve guessed?
+// Missing in some versions of mingw headers
 #ifndef _MSC_VER
-typedef struct _D3DDEVINFO_RESOURCEMANAGER
-{
-  char dummy;
-} D3DDEVINFO_RESOURCEMANAGER, * LPD3DDEVINFO_RESOURCEMANAGER;
+namespace dxvk {
+  typedef struct _D3DDEVINFO_RESOURCEMANAGER
+  {
+    char dummy;
+  } D3DDEVINFO_RESOURCEMANAGER, * LPD3DDEVINFO_RESOURCEMANAGER;
+}
 #endif
 
 // This is the managed pool on D3D9Ex, it's just hidden!
@@ -101,9 +99,7 @@ IDirect3DDevice9On12 : public IUnknown {
   virtual HRESULT STDMETHODCALLTYPE ReturnUnderlyingResource(IDirect3DResource9* resource, UINT num_sync, UINT64* signal_values, ID3D12Fence** fences) = 0;
 };
 
-#ifdef _MSC_VER
-struct __declspec(uuid("e7fda234-b589-4049-940d-8878977531c8")) IDirect3DDevice9On12;
-#else
+#ifndef _MSC_VER
 __CRT_UUID_DECL(IDirect3DDevice9On12,      0xe7fda234,0xb589,0x4049,0x94,0x0d,0x88,0x78,0x97,0x75,0x31,0xc8);
 #endif
 
