@@ -109,6 +109,9 @@ namespace dxvk {
     /// Forced refresh rate, disable other modes
     uint32_t forceRefreshRate;
 
+    /// Restrict the mode count to ensure a maximum total count of 24
+    bool modeCountCompatibility;
+
     /// Always use a spec constant to determine sampler type (instead of just in PS 1.x)
     /// Works around a game bug in Halo CE where it gives cube textures to 2d/volume samplers
     bool forceSamplerTypeSpecConstants;
@@ -116,6 +119,9 @@ namespace dxvk {
     /// Forces sample rate shading
     bool forceSampleRateShading;
 
+    /// Allow D3DLOCK_DISCARD
+    bool allowDiscard;
+
     /// Enumerate adapters by displays
     bool enumerateByDisplays;
 
