@@ -20,11 +20,7 @@ namespace dxvk {
   static constexpr uint32_t SamplerStateCount = D3DSAMP_DMAPOFFSET + 1;
   static constexpr uint32_t SamplerCount      = caps::MaxTexturesPS + caps::MaxTexturesVS + 1;
   static constexpr uint32_t TextureStageStateCount = DXVK_TSS_COUNT;
-
-  namespace hacks::PointSize {
-    static constexpr DWORD AlphaToCoverageDisabled = MAKEFOURCC('A', '2', 'M', '0');
-    static constexpr DWORD AlphaToCoverageEnabled  = MAKEFOURCC('A', '2', 'M', '1');
-  }
+  static constexpr uint32_t PaletteEntryCount = 256;
   
   struct D3D9ClipPlane {
     float coeff[4] = {};
@@ -81,24 +77,20 @@ namespace dxvk {
   };
 
   struct D3D9Light {
-    D3D9Light(const D3DLIGHT9& light, Matrix4 viewMtx) {
-      Diffuse  = Vector4(light.Diffuse.r,  light.Diffuse.g,  light.Diffuse.b,  light.Diffuse.a);
-      Specular = Vector4(light.Specular.r, light.Specular.g, light.Specular.b, light.Specular.a);
-      Ambient  = Vector4(light.Ambient.r,  light.Ambient.g,  light.Ambient.b,  light.Ambient.a);
-
-      Position  = viewMtx * Vector4(light.Position.x,  light.Position.y,  light.Position.z,  1.0f);
-      Direction = Vector4(light.Direction.x, light.Direction.y, light.Direction.z, 0.0f);
-      Direction = normalize(viewMtx * Direction);
-
-      Type         = light.Type;
-      Range        = light.Range;
-      Falloff      = light.Falloff;
-      Attenuation0 = light.Attenuation0;
-      Attenuation1 = light.Attenuation1;
-      Attenuation2 = light.Attenuation2;
-      Theta        = cosf(light.Theta / 2.0f);
-      Phi          = cosf(light.Phi / 2.0f);
-    }
+    D3D9Light(const D3DLIGHT9& light, Matrix4 viewMtx)
+      : Diffuse      ( Vector4(light.Diffuse.r,  light.Diffuse.g,  light.Diffuse.b,  light.Diffuse.a) )
+      , Specular     ( Vector4(light.Specular.r, light.Specular.g, light.Specular.b, light.Specular.a) )
+      , Ambient      ( Vector4(light.Ambient.r,  light.Ambient.g,  light.Ambient.b,  light.Ambient.a) )
+      , Position     ( viewMtx * Vector4(light.Position.x,  light.Position.y,  light.Position.z,  1.0f) )
+      , Direction    ( normalize(viewMtx * Vector4(light.Direction.x, light.Direction.y, light.Direction.z, 0.0f)) )
+      , Type         ( light.Type )
+      , Range        ( light.Range )
+      , Falloff      ( light.Falloff )
+      , Attenuation0 ( light.Attenuation0 )
+      , Attenuation1 ( light.Attenuation1 )
+      , Attenuation2 ( light.Attenuation2 )
+      , Theta        ( cosf(light.Theta / 2.0f) )
+      , Phi          ( cosf(light.Phi / 2.0f) ) { }
 
     Vector4 Diffuse;
     Vector4 Specular;
@@ -236,8 +228,6 @@ namespace dxvk {
   };
 
   struct D3D9SamplerInfo {
-    D3D9SamplerInfo() = default;
-
     D3D9SamplerInfo(const std::array<DWORD, SamplerStateCount>& state)
     : addressU(D3DTEXTUREADDRESS(state[D3DSAMP_ADDRESSU]))
     , addressV(D3DTEXTUREADDRESS(state[D3DSAMP_ADDRESSV]))
@@ -288,6 +278,8 @@ namespace dxvk {
     D3DVIEWPORT9                                        viewport = {};
     RECT                                                scissorRect = {};
 
+    D3DCLIPSTATUS9                                      clipStatus = {0, 0xffffffff};
+
     ItemType<std::array<
       D3D9ClipPlane,
       caps::MaxClipPlanes>>                             clipPlanes = {};
@@ -296,6 +288,11 @@ namespace dxvk {
       std::array<DWORD, TextureStageStateCount>,
       caps::TextureStageCount>>                         textureStages = {};
 
+    std::unordered_map<
+       UINT,
+       std::array<PALETTEENTRY, PaletteEntryCount>>     texturePalettes;
+    UINT                                                texturePaletteNumber = 0u;
+
     ItemType<D3D9ShaderConstantsVSSoftware>             vsConsts;
     ItemType<D3D9ShaderConstantsPS>                     psConsts;
 
@@ -308,9 +305,11 @@ namespace dxvk {
     std::vector<std::optional<D3DLIGHT9>>               lights;
     std::array<DWORD, caps::MaxEnabledLights>           enabledLightIndices;
 
+    float                                               nPatchSegments = 0.0f;
+
     bool IsLightEnabled(DWORD Index) const {
-      const auto& indices = enabledLightIndices;
-      return std::find(indices.begin(), indices.end(), Index) != indices.end();
+      const auto& enabledIndices = enabledLightIndices;
+      return std::find(enabledIndices.begin(), enabledIndices.end(), Index) != enabledIndices.end();
     }
   };
 
