@@ -13,7 +13,7 @@ namespace dxvk {
     const Rc<DxvkAdapter> adapter = device->adapter();
 
     const DxvkDeviceFeatures& devFeatures = device->features();
-    const DxvkDeviceInfo& devInfo = adapter->devicePropertiesExt();
+    const DxvkDeviceInfo& devInfo = device->properties();
 
     useDepthClipWorkaround
       = !devFeatures.extDepthClipEnable.depthClipEnable;
@@ -26,6 +26,9 @@ namespace dxvk {
     supportsTypedUavLoadR32 = (r32Features & VK_FORMAT_FEATURE_2_STORAGE_READ_WITHOUT_FORMAT_BIT);
     supportsRawAccessChains = device->features().nvRawAccessChains.shaderRawAccessChains;
 
+    // Raw access chains are currently broken with byte-address SSBO and descriptor buffers
+    rawAccessChainBug = supportsRawAccessChains && device->canUseDescriptorBuffer();
+
     switch (device->config().useRawSsbo) {
       case Tristate::Auto:  minSsboAlignment = devInfo.core.properties.limits.minStorageBufferOffsetAlignment; break;
       case Tristate::True:  minSsboAlignment =  4u; break;
@@ -33,21 +36,20 @@ namespace dxvk {
     }
     
     invariantPosition        = options.invariantPosition;
-    zeroInitWorkgroupMemory  = options.zeroInitWorkgroupMemory;
     forceVolatileTgsmAccess  = options.forceVolatileTgsmAccess;
     forceComputeUavBarriers  = options.forceComputeUavBarriers;
     disableMsaa              = options.disableMsaa;
     forceSampleRateShading   = options.forceSampleRateShading;
     enableSampleShadingInterlock = device->features().extFragmentShaderInterlock.fragmentShaderSampleInterlock;
-    supportsTightIcbPacking  = device->features().vk12.uniformBufferStandardLayout;
-
-    // Qcom just breaks for no reason if we export point size,
-    // even in an environment where doing so is required.
-    needsPointSizeExport = !device->features().khrMaintenance5.maintenance5
-                        && !device->adapter()->matchesDriver(VK_DRIVER_ID_QUALCOMM_PROPRIETARY);
+    supports16BitPushData    = device->features().vk11.storagePushConstant16;
 
     // ANV up to mesa 25.0.2 breaks when we *don't* explicitly write point size
-    needsPointSizeExport |= device->adapter()->matchesDriver(VK_DRIVER_ID_INTEL_OPEN_SOURCE_MESA, Version(), Version(25, 0, 3));
+    needsPointSizeExport = device->adapter()->matchesDriver(VK_DRIVER_ID_INTEL_OPEN_SOURCE_MESA, Version(), Version(25, 0, 3));
+
+    // Intel's hardware sin/cos is so inaccurate that it causes rendering issues in some games
+    sincosEmulation = device->adapter()->matchesDriver(VK_DRIVER_ID_INTEL_OPEN_SOURCE_MESA)
+                   || device->adapter()->matchesDriver(VK_DRIVER_ID_INTEL_PROPRIETARY_WINDOWS);
+    applyTristate(sincosEmulation, options.sincosEmulation);
 
     // Figure out float control flags to match D3D11 rules
     if (options.floatControls) {
