@@ -248,7 +248,9 @@ namespace dxvk {
     const DxsoDefinedConstants& constants() { return m_constants; }
     uint32_t usedSamplers() const { return m_usedSamplers; }
     uint32_t usedRTs() const { return m_usedRTs; }
-    uint32_t maxDefinedConstant() const { return m_maxDefinedConstant; }
+    int32_t  maxDefinedFloatConstant() const { return m_maxDefinedFloatConstant; }
+    int32_t  maxDefinedIntConstant() const { return m_maxDefinedIntConstant; }
+    int32_t  maxDefinedBoolConstant() const { return m_maxDefinedBoolConstant; }
     uint32_t textureTypes() const { return m_textureTypes; }
 
   private:
@@ -260,7 +262,9 @@ namespace dxvk {
 
     DxsoShaderMetaInfo         m_meta;
     DxsoDefinedConstants       m_constants;
-    uint32_t                   m_maxDefinedConstant = 0u;
+    int32_t                    m_maxDefinedFloatConstant = -1;
+    int32_t                    m_maxDefinedIntConstant = -1;
+    int32_t                    m_maxDefinedBoolConstant = -1;
 
     SpirvModule                m_module;
 
