@@ -3,7 +3,7 @@
 #include <unordered_map>
 #include <vector>
 
-#include "dxvk_descriptor.h"
+#include "dxvk_descriptor_pool.h"
 #include "dxvk_format.h"
 #include "dxvk_hash.h"
 #include "dxvk_memory.h"
@@ -41,35 +41,6 @@ namespace dxvk {
   };
 
 
-  /**
-   * \brief Buffer slice info
-   * 
-   * Stores the Vulkan buffer handle, offset
-   * and length of the slice, and a pointer
-   * to the mapped region..
-   */
-  struct DxvkBufferSliceHandle {
-    VkBuffer      handle = VK_NULL_HANDLE;
-    VkDeviceSize  offset = 0u;
-    VkDeviceSize  length = 0u;
-    void*         mapPtr = nullptr;
-
-    bool eq(const DxvkBufferSliceHandle& other) const {
-      return handle == other.handle
-          && offset == other.offset
-          && length == other.length;
-    }
-
-    size_t hash() const {
-      DxvkHashState result;
-      result.add(std::hash<VkBuffer>()(handle));
-      result.add(std::hash<VkDeviceSize>()(offset));
-      result.add(std::hash<VkDeviceSize>()(length));
-      return result;
-    }
-  };
-
-
   /**
    * \brief Virtual buffer view
    */
@@ -89,15 +60,16 @@ namespace dxvk {
      * \brief Retrieves buffer view handle
      *
      * Creates a new view if the buffer has been invalidated.
+     * \param [in] raw Whether to return a raw or formatted descriptor.
      * \returns Vulkan buffer view handle
      */
-    VkBufferView handle();
+    const DxvkDescriptor* getDescriptor(bool raw);
 
     /**
      * \brief Retrieves buffer slice handle
      * \returns Buffer slice handle
      */
-    DxvkBufferSliceHandle getSliceHandle() const;
+    DxvkResourceBufferInfo getSliceInfo() const;
 
     /**
      * \brief Element count
@@ -141,7 +113,11 @@ namespace dxvk {
     DxvkBufferViewKey m_key     = { };
 
     uint32_t          m_version = 0u;
-    VkBufferView      m_handle  = VK_NULL_HANDLE;
+
+    const DxvkDescriptor* m_raw       = nullptr;
+    const DxvkDescriptor* m_formatted = nullptr;
+
+    void updateViews();
 
   };
 
@@ -212,16 +188,6 @@ namespace dxvk {
         : nullptr;
     }
 
-    /**
-     * \brief GPU address
-     *
-     * May be 0 if the device address usage flag is not
-     * enabled for this buffer.
-     */
-    VkDeviceAddress gpuAddress() const {
-      return m_bufferInfo.gpuAddress;
-    }
-
     /**
      * \brief Queries shader stages that can access this buffer
      *
@@ -236,13 +202,8 @@ namespace dxvk {
      * \brief Retrieves slice handle
      * \returns Buffer slice handle
      */
-    DxvkBufferSliceHandle getSliceHandle() const {
-      DxvkBufferSliceHandle result = { };
-      result.handle = m_bufferInfo.buffer;
-      result.offset = m_bufferInfo.offset;
-      result.length = m_info.size;
-      result.mapPtr = mapPtr(0);
-      return result;
+    DxvkResourceBufferInfo getSliceInfo() const {
+      return getSliceInfo(0u, m_info.size);
     }
 
     /**
@@ -252,24 +213,25 @@ namespace dxvk {
      * \param [in] length Sub slice length
      * \returns Buffer slice handle
      */
-    DxvkBufferSliceHandle getSliceHandle(VkDeviceSize offset, VkDeviceSize length) const {
-      DxvkBufferSliceHandle result = { };
-      result.handle = m_bufferInfo.buffer;
+    DxvkResourceBufferInfo getSliceInfo(VkDeviceSize offset, VkDeviceSize length) const {
+      DxvkResourceBufferInfo result;
+      result.buffer = m_bufferInfo.buffer;
       result.offset = m_bufferInfo.offset + offset;
-      result.length = length;
+      result.size = length;
       result.mapPtr = mapPtr(offset);
+      result.gpuAddress = m_bufferInfo.gpuAddress + offset;
       return result;
     }
 
     /**
      * \brief Retrieves descriptor info
-     * 
+     *
      * \param [in] offset Buffer slice offset
      * \param [in] length Buffer slice length
      * \returns Buffer slice descriptor
      */
-    DxvkDescriptorInfo getDescriptor(VkDeviceSize offset, VkDeviceSize length) const {
-      DxvkDescriptorInfo result = { };
+    DxvkLegacyDescriptor getDescriptor(VkDeviceSize offset, VkDeviceSize length) const {
+      DxvkLegacyDescriptor result = { };
       result.buffer.buffer = m_bufferInfo.buffer;
       result.buffer.offset = m_bufferInfo.offset + offset;
       result.buffer.range = length;
@@ -347,6 +309,15 @@ namespace dxvk {
       if (unlikely(m_info.debugName))
         updateDebugName();
 
+      // If this is a device-local buffer, update residency
+      if (!(m_properties & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)) {
+        auto common = m_properties & m_storage->getMemoryProperties();
+
+        updateResidencyStatus((common & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)
+          ? DxvkResourceResidency::Resident
+          : DxvkResourceResidency::Evicted);
+      }
+
       // Implicitly invalidate views
       m_version += 1u;
       return result;
@@ -364,9 +335,9 @@ namespace dxvk {
      * \brief Retrieves resource ID for barrier tracking
      * \returns Unique resource ID
      */
-    uint64_t getResourceId() const {
+    bit::uint48_t getResourceId() const {
       constexpr static size_t Align = alignof(DxvkResourceAllocation);
-      return reinterpret_cast<uintptr_t>(m_storage.ptr()) / (Align & -Align);
+      return bit::uint48_t(reinterpret_cast<uintptr_t>(m_storage.ptr()) / (Align & -Align));
     }
 
     /**
@@ -418,10 +389,17 @@ namespace dxvk {
      */
     void setDebugName(const char* name);
 
+    /**
+     * \brief Retrieves debug name
+     * \returns Debug name
+     */
+    const char* getDebugName() const {
+      return m_debugName.c_str();
+    }
+
   private:
 
     Rc<vk::DeviceFn>            m_vkd;
-    DxvkMemoryAllocator*        m_allocator     = nullptr;
     VkMemoryPropertyFlags       m_properties    = 0u;
     VkShaderStageFlags          m_shaderStages  = 0u;
     DxvkSharingModeInfo         m_sharingMode   = { };
@@ -560,10 +538,10 @@ namespace dxvk {
      * Returns the buffer handle and offset
      * \returns Buffer slice handle
      */
-    DxvkBufferSliceHandle getSliceHandle() const {
-      return m_buffer != nullptr
-        ? m_buffer->getSliceHandle(m_offset, m_length)
-        : DxvkBufferSliceHandle();
+    DxvkResourceBufferInfo getSliceInfo() const {
+      return m_buffer
+        ? m_buffer->getSliceInfo(m_offset, m_length)
+        : DxvkResourceBufferInfo();
     }
 
     /**
@@ -573,20 +551,20 @@ namespace dxvk {
      * \param [in] length Sub slice length
      * \returns Buffer slice handle
      */
-    DxvkBufferSliceHandle getSliceHandle(VkDeviceSize offset, VkDeviceSize length) const {
-      return m_buffer != nullptr
-        ? m_buffer->getSliceHandle(m_offset + offset, length)
-        : DxvkBufferSliceHandle();
+    DxvkResourceBufferInfo getSliceInfo(VkDeviceSize offset, VkDeviceSize length) const {
+      return m_buffer
+        ? m_buffer->getSliceInfo(m_offset + offset, length)
+        : DxvkResourceBufferInfo();
     }
 
     /**
      * \brief Retrieves descriptor info
      * \returns Buffer slice descriptor
      */
-    DxvkDescriptorInfo getDescriptor() const {
+    DxvkLegacyDescriptor getDescriptor() const {
       return m_buffer->getDescriptor(m_offset, m_length);
     }
-    
+
     /**
      * \brief Pointer to mapped memory region
      * 
@@ -660,28 +638,26 @@ namespace dxvk {
 
 
 
-  inline VkBufferView DxvkBufferView::handle() {
-    if (likely(m_version == m_buffer->m_version))
-      return m_handle;
+  inline const DxvkDescriptor* DxvkBufferView::getDescriptor(bool raw) {
+    if (unlikely(m_version < m_buffer->m_version))
+      updateViews();
 
-    m_handle = m_buffer->m_storage->createBufferView(m_key);
-    m_version = m_buffer->m_version;
-    return m_handle;
+    return raw ? m_raw : m_formatted;
   }
 
 
-  inline DxvkBufferSliceHandle DxvkBufferView::getSliceHandle() const {
-    return m_buffer->getSliceHandle(m_key.offset, m_key.size);
+  inline DxvkResourceBufferInfo DxvkBufferView::getSliceInfo() const {
+    return m_buffer->getSliceInfo(m_key.offset, m_key.size);
   }
 
 
-  inline void DxvkBufferView::incRef() {
+  force_inline void DxvkBufferView::incRef() {
     m_buffer->incRef();
   }
 
 
-  inline void DxvkBufferView::decRef() {
+  force_inline void DxvkBufferView::decRef() {
     m_buffer->decRef();
   }
 
-}
\ No newline at end of file
+}
