@@ -19,7 +19,7 @@ namespace dxvk {
     m_debugFlags        (instance->debugFlags()),
     m_queues            (queues),
     m_features          (features),
-    m_properties        (adapter->devicePropertiesExt()),
+    m_properties        (adapter->deviceProperties()),
     m_perfHints         (getPerfHints()),
     m_objects           (this),
     m_submissionQueue   (this, queueCallback) {
@@ -70,33 +70,16 @@ namespace dxvk {
       info.pNext = &formatList;
     }
 
-    if (m_features.khrMaintenance5.maintenance5) {
-      VkImageSubresource2KHR subresourceInfo = { VK_STRUCTURE_TYPE_IMAGE_SUBRESOURCE_2_KHR };
-      subresourceInfo.imageSubresource = subresource;
+    VkImageSubresource2KHR subresourceInfo = { VK_STRUCTURE_TYPE_IMAGE_SUBRESOURCE_2_KHR };
+    subresourceInfo.imageSubresource = subresource;
 
-      VkDeviceImageSubresourceInfoKHR query = { VK_STRUCTURE_TYPE_DEVICE_IMAGE_SUBRESOURCE_INFO_KHR };
-      query.pCreateInfo = &info;
-      query.pSubresource = &subresourceInfo;
+    VkDeviceImageSubresourceInfoKHR query = { VK_STRUCTURE_TYPE_DEVICE_IMAGE_SUBRESOURCE_INFO_KHR };
+    query.pCreateInfo = &info;
+    query.pSubresource = &subresourceInfo;
 
-      VkSubresourceLayout2KHR layout = { VK_STRUCTURE_TYPE_SUBRESOURCE_LAYOUT_2_KHR };
-      m_vkd->vkGetDeviceImageSubresourceLayoutKHR(m_vkd->device(), &query, &layout);
-      return layout.subresourceLayout;
-    } else {
-      // Technically, there is no guarantee that all images with the same
-      // properties are going to have consistent subresource layouts if
-      // maintenance5 is not supported, but the only such use case we care
-      // about is RenderDoc.
-      VkImage image = VK_NULL_HANDLE;
-      VkResult vr = m_vkd->vkCreateImage(m_vkd->device(), &info, nullptr, &image);
-
-      if (vr != VK_SUCCESS)
-        throw DxvkError(str::format("Failed to create temporary image: ", vr));
-
-      VkSubresourceLayout layout = { };
-      m_vkd->vkGetImageSubresourceLayout(m_vkd->device(), image, &subresource, &layout);
-      m_vkd->vkDestroyImage(m_vkd->device(), image, nullptr);
-      return layout;
-    }
+    VkSubresourceLayout2KHR layout = { VK_STRUCTURE_TYPE_SUBRESOURCE_LAYOUT_2_KHR };
+    m_vkd->vkGetDeviceImageSubresourceLayoutKHR(m_vkd->device(), &query, &layout);
+    return layout.subresourceLayout;
   }
 
 
@@ -245,6 +228,227 @@ namespace dxvk {
   }
 
 
+  const DxvkPipelineLayout* DxvkDevice::createBuiltInPipelineLayout(
+          DxvkPipelineLayoutFlags         flags,
+          VkShaderStageFlags              pushDataStages,
+          VkDeviceSize                    pushDataSize,
+          uint32_t                        bindingCount,
+    const DxvkDescriptorSetLayoutBinding* bindings) {
+    DxvkPipelineLayoutKey key(DxvkPipelineLayoutType::Merged, flags);
+
+    if (pushDataSize) {
+      key.addStages(pushDataStages);
+
+      DxvkPushDataBlock pushData(pushDataStages,
+        0u, pushDataSize, sizeof(uint32_t), 0u);
+
+      key.addPushData(pushData);
+    }
+
+    if (bindingCount) {
+      DxvkDescriptorSetLayoutKey setLayoutKey;
+
+      for (uint32_t i = 0; i < bindingCount; i++) {
+        key.addStages(bindings[i].getStageMask());
+        setLayoutKey.add(bindings[i]);
+      }
+
+      const auto* layout = m_objects.pipelineManager().createDescriptorSetLayout(setLayoutKey);
+      key.setDescriptorSetLayouts(1, &layout);
+    }
+
+    return m_objects.pipelineManager().createPipelineLayout(key);
+  }
+
+
+  VkPipeline DxvkDevice::createBuiltInComputePipeline(
+    const DxvkPipelineLayout*             layout,
+    const util::DxvkBuiltInShaderStage&   stage) {
+    VkShaderModuleCreateInfo moduleInfo = { VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO };
+    moduleInfo.codeSize = stage.size;
+    moduleInfo.pCode = stage.code;
+
+    VkPipelineCreateFlags2CreateInfo pipelineFlags = { VK_STRUCTURE_TYPE_PIPELINE_CREATE_FLAGS_2_CREATE_INFO };
+
+    if (canUseDescriptorBuffer())
+      pipelineFlags.flags |= VK_PIPELINE_CREATE_2_DESCRIPTOR_BUFFER_BIT_EXT;
+
+    VkComputePipelineCreateInfo pipelineInfo = { VK_STRUCTURE_TYPE_COMPUTE_PIPELINE_CREATE_INFO, &pipelineFlags };
+    pipelineInfo.layout = layout->getPipelineLayout();
+    pipelineInfo.basePipelineIndex = -1;
+
+    VkPipelineShaderStageCreateInfo& stageInfo = pipelineInfo.stage;
+    stageInfo = { VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, &moduleInfo };
+    stageInfo.stage = VK_SHADER_STAGE_COMPUTE_BIT;
+    stageInfo.pName = "main";
+    stageInfo.pSpecializationInfo = stage.spec;
+
+    VkPipeline pipeline = VK_NULL_HANDLE;
+
+    VkResult vr = m_vkd->vkCreateComputePipelines(m_vkd->device(),
+      VK_NULL_HANDLE, 1, &pipelineInfo, nullptr, &pipeline);
+
+    if (vr)
+      throw DxvkError(str::format("Failed to create built-in compute pipeline: ", vr));
+
+    return pipeline;
+  }
+
+
+  VkPipeline DxvkDevice::createBuiltInGraphicsPipeline(
+    const DxvkPipelineLayout*             layout,
+    const util::DxvkBuiltInGraphicsState& state) {
+    constexpr size_t MaxStages = 3u;
+
+    // Build shader stage infos
+    small_vector<std::pair<VkShaderStageFlagBits, util::DxvkBuiltInShaderStage>, MaxStages> stages;
+
+    if (state.vs.code) stages.push_back({ VK_SHADER_STAGE_VERTEX_BIT,   state.vs });
+    if (state.gs.code) stages.push_back({ VK_SHADER_STAGE_GEOMETRY_BIT, state.gs });
+    if (state.fs.code) stages.push_back({ VK_SHADER_STAGE_FRAGMENT_BIT, state.fs });
+
+    small_vector<VkShaderModuleCreateInfo, MaxStages> moduleInfos;
+
+    for (size_t i = 0; i < stages.size(); i++) {
+      auto& info = moduleInfos.emplace_back();
+      info.sType = VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO;
+      info.codeSize = stages[i].second.size;
+      info.pCode = stages[i].second.code;
+    }
+
+    small_vector<VkPipelineShaderStageCreateInfo, MaxStages> stageInfos;
+
+    for (size_t i = 0; i < stages.size(); i++) {
+      auto& info = stageInfos.emplace_back();
+      info.sType = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO;
+      info.pNext = &moduleInfos[i];
+      info.stage = stages[i].first;
+      info.pName = "main";
+      info.pSpecializationInfo = stages[i].second.spec;
+    }
+
+    // Attachment format infos, useful to set up state
+    auto depthFormatInfo = lookupFormatInfo(state.depthFormat);
+
+    // Default vertex input state
+    VkPipelineVertexInputStateCreateInfo viState = { VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO };
+
+    // Default input assembly state using triangle list
+    VkPipelineInputAssemblyStateCreateInfo iaState = { VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO };
+    iaState.topology = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST;
+
+    // Default viewport state, needs to be defined even if everything is dynamic
+    VkPipelineViewportStateCreateInfo vpState = { VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO };
+
+    // Default rasterization state
+    VkPipelineRasterizationStateCreateInfo rsState = { VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO };
+    rsState.cullMode          = VK_CULL_MODE_NONE;
+    rsState.frontFace         = VK_FRONT_FACE_COUNTER_CLOCKWISE;
+    rsState.polygonMode       = VK_POLYGON_MODE_FILL;
+    rsState.depthClampEnable  = state.depthFormat != VK_FORMAT_UNDEFINED;
+    rsState.lineWidth         = 1.0f;
+
+    // Multisample state. Enables rendering to all samples at once.
+    uint32_t sampleMask = (1u << uint32_t(state.sampleCount)) - 1u;
+
+    VkPipelineMultisampleStateCreateInfo msState = { VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO };
+    msState.rasterizationSamples  = state.sampleCount;
+    msState.pSampleMask           = &sampleMask;
+    msState.sampleShadingEnable   = VK_FALSE;
+    msState.minSampleShading      = 1.0f;
+
+    // Default depth-stencil state, enables depth and stencil write-through
+    VkPipelineDepthStencilStateCreateInfo dsState = { VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO };
+
+    if (state.depthFormat && (depthFormatInfo->aspectMask & VK_IMAGE_ASPECT_DEPTH_BIT)) {
+      dsState.depthTestEnable   = VK_TRUE;
+      dsState.depthWriteEnable  = VK_TRUE;
+      dsState.depthCompareOp    = VK_COMPARE_OP_ALWAYS;
+    }
+
+    if (state.depthFormat && (depthFormatInfo->aspectMask & VK_IMAGE_ASPECT_STENCIL_BIT)) {
+      VkStencilOpState stencil = { };
+      stencil.passOp      = VK_STENCIL_OP_REPLACE;
+      stencil.failOp      = VK_STENCIL_OP_REPLACE;
+      stencil.depthFailOp = VK_STENCIL_OP_REPLACE;
+      stencil.compareOp   = VK_COMPARE_OP_ALWAYS;
+      stencil.compareMask = 0xffffffffu;
+      stencil.writeMask   = 0xffffffffu;
+
+      dsState.stencilTestEnable = VK_TRUE;
+      dsState.front       = stencil;
+      dsState.back        = stencil;
+    }
+
+    // Default blend state, only used if color attachments are present
+    VkPipelineColorBlendAttachmentState cbAttachment = { };
+    cbAttachment.colorWriteMask = VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT
+                                | VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
+
+    VkPipelineColorBlendStateCreateInfo cbState = { VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO };
+
+    if (state.colorFormat) {
+      cbState.attachmentCount = 1u;
+      cbState.pAttachments = state.cbAttachment ? state.cbAttachment : &cbAttachment;
+    }
+
+    // Prepare dynamic states
+    small_vector<VkDynamicState, 4> dynamicStates;
+    dynamicStates.push_back(VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT);
+    dynamicStates.push_back(VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT);
+
+    for (uint32_t i = 0; i < state.dynamicStateCount; i++)
+      dynamicStates.push_back(state.dynamicStates[i]);
+
+    VkPipelineDynamicStateCreateInfo dyState = { VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO };
+    dyState.dynamicStateCount = dynamicStates.size();
+    dyState.pDynamicStates = dynamicStates.data();
+
+    // Build rendering attachment info
+    VkPipelineRenderingCreateInfo renderingInfo = { VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO };
+
+    if (state.colorFormat) {
+      renderingInfo.colorAttachmentCount = 1u;
+      renderingInfo.pColorAttachmentFormats = &state.colorFormat;
+    }
+
+    if (state.depthFormat && (depthFormatInfo->aspectMask & VK_IMAGE_ASPECT_DEPTH_BIT))
+      renderingInfo.depthAttachmentFormat = state.depthFormat;
+
+    if (state.depthFormat && (depthFormatInfo->aspectMask & VK_IMAGE_ASPECT_STENCIL_BIT))
+      renderingInfo.stencilAttachmentFormat = state.depthFormat;
+
+    VkPipelineCreateFlags2CreateInfo pipelineFlags = { VK_STRUCTURE_TYPE_PIPELINE_CREATE_FLAGS_2_CREATE_INFO, &renderingInfo };
+
+    if (canUseDescriptorBuffer())
+      pipelineFlags.flags |= VK_PIPELINE_CREATE_2_DESCRIPTOR_BUFFER_BIT_EXT;
+
+    VkGraphicsPipelineCreateInfo pipelineInfo = { VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO, &pipelineFlags };
+    pipelineInfo.stageCount = stageInfos.size();
+    pipelineInfo.pStages = stageInfos.data();
+    pipelineInfo.pVertexInputState = state.viState ? state.viState : &viState;
+    pipelineInfo.pInputAssemblyState = state.iaState ? state.iaState : &iaState;
+    pipelineInfo.pViewportState = &vpState;
+    pipelineInfo.pRasterizationState = state.rsState ? state.rsState : &rsState;
+    pipelineInfo.pMultisampleState = &msState;
+    pipelineInfo.pDepthStencilState = state.depthFormat ? (state.dsState ? state.dsState : &dsState) : nullptr;
+    pipelineInfo.pColorBlendState = state.colorFormat ? &cbState : nullptr;
+    pipelineInfo.pDynamicState = &dyState;
+    pipelineInfo.layout = layout->getPipelineLayout();
+    pipelineInfo.basePipelineIndex = -1;
+
+    VkPipeline pipeline = VK_NULL_HANDLE;
+
+    VkResult vr = m_vkd->vkCreateGraphicsPipelines(m_vkd->device(),
+      VK_NULL_HANDLE, 1, &pipelineInfo, nullptr, &pipeline);
+
+    if (vr)
+      throw DxvkError(str::format("Failed to create built-in graphics pipeline: ", vr));
+
+    return pipeline;
+  }
+
+
   DxvkStatCounters DxvkDevice::getStatCounters() {
     DxvkPipelineCount pipe = m_objects.pipelineManager().getPipelineCount();
     DxvkPipelineWorkerStats workers = m_objects.pipelineManager().getWorkerStats();
