@@ -15,11 +15,12 @@ namespace dxvk {
   enum class DxvkDeviceFilterFlag {
     MatchDeviceName   = 0,
     SkipCpuDevices    = 1,
+    MatchDeviceUUID   = 3
   };
-  
+
   using DxvkDeviceFilterFlags = Flags<DxvkDeviceFilterFlag>;
-  
-  
+
+
   /**
    * \brief DXVK device filter
    * 
@@ -29,30 +30,30 @@ namespace dxvk {
    * device.
    */
   class DxvkDeviceFilter {
-    
+
   public:
-    
+
     DxvkDeviceFilter(
             DxvkDeviceFilterFlags flags,
       const DxvkOptions&          options);
 
     ~DxvkDeviceFilter();
-    
+
     /**
      * \brief Tests an adapter
-     * 
-     * \param [in] properties Adapter properties
-     * \returns \c true if the test passes
+     *
+     * \param [in] adapter Adapter object
+     * \returns \c true if the device can be used
      */
-    bool testAdapter(
-      const VkPhysicalDeviceProperties& properties) const;
-    
+    bool testAdapter(DxvkAdapter& adapter) const;
+
   private:
-    
+
     DxvkDeviceFilterFlags m_flags;
-    
+
     std::string m_matchDeviceName;
-    
+    std::string m_matchDeviceUUID;
+
   };
-  
-}
\ No newline at end of file
+
+}
