@@ -2,6 +2,8 @@
 
 #include "../util/config/config.h"
 
+#include "../vulkan/vulkan_loader.h"
+
 namespace dxvk {
 
   struct DxvkOptions {
@@ -11,9 +13,6 @@ namespace dxvk {
     /// Enable debug utils
     bool enableDebugUtils = false;
 
-    /// Enable state cache
-    bool enableStateCache = true;
-
     /// Enable memory defragmentation
     Tristate enableMemoryDefrag = Tristate::Auto;
 
@@ -24,6 +23,9 @@ namespace dxvk {
     /// Enable graphics pipeline library
     Tristate enableGraphicsPipelineLibrary = Tristate::Auto;
 
+    /// Enable descriptor buffer
+    Tristate enableDescriptorBuffer = Tristate::Auto;
+
     /// Enables pipeline lifetime tracking
     Tristate trackPipelineLifetime = Tristate::Auto;
 
@@ -37,6 +39,16 @@ namespace dxvk {
     /// or FIFO_RELAXED (if false) present mode
     Tristate tearFree = Tristate::Auto;
 
+    /// Enables latency sleep
+    Tristate latencySleep = Tristate::Auto;
+
+    /// Latency tolerance, in microseconds
+    int32_t latencyTolerance = 0u;
+
+    /// Disable VK_NV_low_latency2. This extension
+    /// appears to be all sorts of broken on 32-bit.
+    Tristate disableNvLowLatency2 = Tristate::Auto;
+
     // Hides integrated GPUs if dedicated GPUs are
     // present. May be necessary for some games that
     // incorrectly assume monitor layouts.
@@ -48,6 +60,12 @@ namespace dxvk {
     /// Allows full-screen exclusive mode on Windows
     bool allowFse = false;
 
+    /// Whether to enable tiler optimizations
+    Tristate tilerMode = Tristate::Auto;
+
+    /// Overrides memory budget for DXVK
+    VkDeviceSize maxMemoryBudget = 0u;
+
     // Device name
     std::string deviceFilter;
   };
