@@ -1,20 +1,22 @@
 #include "dxvk_swapchain_blitter.h"
 
+#include <dxvk_cursor_frag.h>
+#include <dxvk_cursor_vert.h>
 #include <dxvk_present_frag.h>
 #include <dxvk_present_frag_blit.h>
 #include <dxvk_present_frag_ms.h>
-#include <dxvk_present_frag_ms_amd.h>
 #include <dxvk_present_frag_ms_blit.h>
 #include <dxvk_present_vert.h>
 
 namespace dxvk {
   
-  DxvkSwapchainBlitter::DxvkSwapchainBlitter(const Rc<DxvkDevice>& device)
-  : m_device(device),
-    m_setLayout(createSetLayout()),
-    m_pipelineLayout(createPipelineLayout()) {
+  DxvkSwapchainBlitter::DxvkSwapchainBlitter(
+    const Rc<DxvkDevice>& device,
+    const Rc<hud::Hud>&   hud)
+  : m_device(device), m_hud(hud),
+    m_blitLayout(createBlitPipelineLayout()),
+    m_cursorLayout(createCursorPipelineLayout()) {
     this->createSampler();
-    this->createShaders();
   }
 
 
@@ -24,27 +26,23 @@ namespace dxvk {
     for (const auto& p : m_pipelines)
       vk->vkDestroyPipeline(vk->device(), p.second, nullptr);
 
-    vk->vkDestroyShaderModule(vk->device(), m_shaderVsBlit.stageInfo.module, nullptr);
-    vk->vkDestroyShaderModule(vk->device(), m_shaderFsBlit.stageInfo.module, nullptr);
-    vk->vkDestroyShaderModule(vk->device(), m_shaderFsCopy.stageInfo.module, nullptr);
-    vk->vkDestroyShaderModule(vk->device(), m_shaderFsMsBlit.stageInfo.module, nullptr);
-    vk->vkDestroyShaderModule(vk->device(), m_shaderFsMsResolve.stageInfo.module, nullptr);
-
-    vk->vkDestroyPipelineLayout(vk->device(), m_pipelineLayout, nullptr);
-    vk->vkDestroyDescriptorSetLayout(vk->device(), m_setLayout, nullptr);
+    for (const auto& p : m_cursorPipelines)
+      vk->vkDestroyPipeline(vk->device(), p.second, nullptr);
   }
 
 
-  void DxvkSwapchainBlitter::beginPresent(
-    const DxvkContextObjects& ctx,
+  void DxvkSwapchainBlitter::present(
+    const Rc<DxvkCommandList>&ctx,
     const Rc<DxvkImageView>&  dstView,
-          VkColorSpaceKHR     dstColorSpace,
           VkRect2D            dstRect,
     const Rc<DxvkImageView>&  srcView,
-          VkColorSpaceKHR     srcColorSpace,
           VkRect2D            srcRect) {
     std::unique_lock lock(m_mutex);
 
+    // Update HUD, if we have one
+    if (m_hud)
+      m_hud->update();
+
     // Fix up default present areas if necessary
     if (!dstRect.extent.width || !dstRect.extent.height) {
       dstRect.offset = { 0, 0 };
@@ -66,11 +64,21 @@ namespace dxvk {
     if (m_cursorBuffer)
       uploadCursorImage(ctx);
 
+    // If we can't do proper blending, render the HUD into a separate image
+    bool composite = needsComposition(dstView);
+
+    if (m_hud && composite)
+      renderHudImage(ctx, dstView->mipLevelExtent(0));
+    else
+      destroyHudImage();
+
+    VkImageLayout renderLayout = dstView->getLayout();
+
     VkImageMemoryBarrier2 barrier = { VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER_2 };
     barrier.dstAccessMask = VK_ACCESS_2_COLOR_ATTACHMENT_WRITE_BIT;
     barrier.dstStageMask = VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT;
     barrier.oldLayout = VK_IMAGE_LAYOUT_UNDEFINED;
-    barrier.newLayout = dstView->image()->pickLayout(VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL);
+    barrier.newLayout = renderLayout;
     barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
     barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
     barrier.image = dstView->image()->handle();
@@ -80,13 +88,13 @@ namespace dxvk {
     depInfo.imageMemoryBarrierCount = 1;
     depInfo.pImageMemoryBarriers = &barrier;
 
-    ctx.cmd->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
+    ctx->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
 
     VkExtent3D dstExtent = dstView->mipLevelExtent(0u);
 
     VkRenderingAttachmentInfo attachmentInfo = { VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO };
     attachmentInfo.imageView = dstView->handle();
-    attachmentInfo.imageLayout = dstView->image()->pickLayout(VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL);
+    attachmentInfo.imageLayout = renderLayout;
     attachmentInfo.loadOp = VK_ATTACHMENT_LOAD_OP_DONT_CARE;
     attachmentInfo.storeOp = VK_ATTACHMENT_STORE_OP_STORE;
 
@@ -100,50 +108,38 @@ namespace dxvk {
     renderInfo.colorAttachmentCount = 1;
     renderInfo.pColorAttachments = &attachmentInfo;
 
-    ctx.cmd->cmdBeginRendering(&renderInfo);
+    ctx->cmdBeginRendering(&renderInfo);
 
-    performDraw(ctx,
-      dstView, dstColorSpace, dstRect,
-      srcView, srcColorSpace, srcRect,
-      VK_FALSE);
-  }
+    performDraw(ctx, dstView, dstRect,
+      srcView, srcRect, composite);
 
+    if (!composite) {
+      if (m_hud)
+        m_hud->render(ctx, dstView);
 
-  void DxvkSwapchainBlitter::endPresent(
-    const DxvkContextObjects& ctx,
-    const Rc<DxvkImageView>&  dstView,
-          VkColorSpaceKHR     dstColorSpace) {
-    std::unique_lock lock(m_mutex);
-
-    if (m_cursorView) {
-      VkRect2D cursorArea = { };
-      cursorArea.extent.width = m_cursorImage->info().extent.width;
-      cursorArea.extent.height = m_cursorImage->info().extent.height;
-
-      performDraw(ctx, dstView, dstColorSpace, m_cursorRect,
-        m_cursorView, VK_COLOR_SPACE_SRGB_NONLINEAR_KHR, cursorArea,
-        VK_TRUE);
+      if (m_cursorView)
+        renderCursor(ctx, dstView);
     }
 
-    ctx.cmd->cmdEndRendering();
+    ctx->cmdEndRendering();
 
-    VkImageMemoryBarrier2 barrier = { VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER_2 };
+    barrier = { VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER_2 };
     barrier.srcAccessMask = VK_ACCESS_2_COLOR_ATTACHMENT_WRITE_BIT;
     barrier.srcStageMask = VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT;
     barrier.dstAccessMask = VK_ACCESS_2_MEMORY_READ_BIT;
     barrier.dstStageMask = VK_PIPELINE_STAGE_2_ALL_COMMANDS_BIT;
-    barrier.oldLayout = dstView->image()->pickLayout(VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL);
+    barrier.oldLayout = renderLayout;
     barrier.newLayout = dstView->image()->info().layout;
     barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
     barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
     barrier.image = dstView->image()->handle();
     barrier.subresourceRange = dstView->imageSubresources();
 
-    VkDependencyInfo depInfo = { VK_STRUCTURE_TYPE_DEPENDENCY_INFO };
+    depInfo = { VK_STRUCTURE_TYPE_DEPENDENCY_INFO };
     depInfo.imageMemoryBarrierCount = 1;
     depInfo.pImageMemoryBarriers = &barrier;
 
-    ctx.cmd->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
+    ctx->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
   }
 
 
@@ -212,6 +208,7 @@ namespace dxvk {
                        | VK_ACCESS_SHADER_READ_BIT;
       imageInfo.tiling = VK_IMAGE_TILING_OPTIMAL;
       imageInfo.layout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
+      imageInfo.colorSpace = VK_COLOR_SPACE_SRGB_NONLINEAR_KHR;
       imageInfo.debugName = "Swapchain cursor";
 
       m_cursorImage = m_device->createImage(imageInfo, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
@@ -244,16 +241,17 @@ namespace dxvk {
 
 
   void DxvkSwapchainBlitter::performDraw(
-    const DxvkContextObjects& ctx,
+    const Rc<DxvkCommandList>&ctx,
     const Rc<DxvkImageView>&  dstView,
-          VkColorSpaceKHR     dstColorSpace,
           VkRect2D            dstRect,
     const Rc<DxvkImageView>&  srcView,
-          VkColorSpaceKHR     srcColorSpace,
           VkRect2D            srcRect,
-          VkBool32            enableBlending) {
-    if (unlikely(m_device->isDebugEnabled())) {
-      ctx.cmd->cmdInsertDebugUtilsLabel(DxvkCmdBuffer::ExecBuffer,
+          VkBool32            composite) {
+    VkColorSpaceKHR dstColorSpace = dstView->image()->info().colorSpace;
+    VkColorSpaceKHR srcColorSpace = srcView->image()->info().colorSpace;
+
+    if (unlikely(m_device->debugFlags().test(DxvkDebugFlag::Capture))) {
+      ctx->cmdBeginDebugUtilsLabel(DxvkCmdBuffer::ExecBuffer,
         vk::makeLabel(0xdcc0f0, "Swapchain blit"));
     }
 
@@ -281,14 +279,14 @@ namespace dxvk {
     viewport.minDepth = 0.0f;
     viewport.maxDepth = 0.0f;
 
-    ctx.cmd->cmdSetViewport(1, &viewport);
+    ctx->cmdSetViewport(1, &viewport);
 
     VkRect2D scissor = { };
     scissor.offset = coordA;
     scissor.extent.width = uint32_t(coordB.x - coordA.x);
     scissor.extent.height = uint32_t(coordB.y - coordA.y);
 
-    ctx.cmd->cmdSetScissor(1, &scissor);
+    ctx->cmdSetScissor(1, &scissor);
 
     DxvkSwapchainPipelineKey key;
     key.srcSpace = srcColorSpace;
@@ -298,67 +296,273 @@ namespace dxvk {
     key.dstFormat = dstView->info().format;
     key.needsGamma = m_gammaView != nullptr;
     key.needsBlit = dstRect.extent != srcRect.extent;
-    key.needsBlending = enableBlending;
+    key.compositeHud = composite && m_hudSrv;
+    key.compositeCursor = composite && m_cursorView;
 
-    VkPipeline pipeline = getPipeline(key);
+    VkPipeline pipeline = getBlitPipeline(key);
 
-    ctx.cmd->cmdBindPipeline(DxvkCmdBuffer::ExecBuffer,
-      VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline);
+    // Set up resource bindings
+    std::array<DxvkDescriptorWrite, 4> descriptors = { };
 
-    VkDescriptorSet set = ctx.descriptorPool->alloc(m_setLayout);
+    auto& imageDescriptor = descriptors[0u];
+    imageDescriptor.descriptorType = VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE;
+    imageDescriptor.descriptor = srcView->getDescriptor();
 
-    VkDescriptorImageInfo imageDescriptor = { };
-    imageDescriptor.sampler = m_samplerPresent->handle();
-    imageDescriptor.imageView = srcView->handle();
-    imageDescriptor.imageLayout = srcView->image()->info().layout;
+    auto& gammaDescriptor = descriptors[1u];
+    gammaDescriptor.descriptorType = VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE;
 
-    VkDescriptorImageInfo gammaDescriptor = { };
-    gammaDescriptor.sampler = m_samplerGamma->handle();
+    if (m_gammaView)
+      gammaDescriptor.descriptor = m_gammaView->getDescriptor();
 
-    if (m_gammaView) {
-      gammaDescriptor.imageView = m_gammaView->handle();
-      gammaDescriptor.imageLayout = m_gammaView->image()->info().layout;
+    auto& hudDescriptor = descriptors[2u];
+    hudDescriptor.descriptorType = VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE;
+
+    if (m_hudSrv)
+      hudDescriptor.descriptor = m_hudSrv->getDescriptor();
+
+    auto& cursorDescriptor = descriptors[3u];
+    cursorDescriptor.descriptorType = VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE;
+
+    uint32_t cursorSampler = m_samplerCursorNearest->getDescriptor().samplerIndex;
+
+    if (m_cursorView) {
+      VkExtent3D extent = m_cursorImage->info().extent;
+
+      if (m_cursorRect.extent.width != extent.width
+       || m_cursorRect.extent.height != extent.height)
+        cursorSampler = m_samplerCursorLinear->getDescriptor().samplerIndex;
+
+      cursorDescriptor.descriptor = m_cursorView->getDescriptor();
     }
 
-    std::array<VkWriteDescriptorSet, 2> descriptorWrites = {{
-      { VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET, nullptr,
-        set, 0, 0, 1, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, &imageDescriptor },
-      { VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET, nullptr,
-        set, 1, 0, 1, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, &gammaDescriptor },
-    }};
-
-    ctx.cmd->updateDescriptorSets(
-      descriptorWrites.size(), descriptorWrites.data());
-
-    ctx.cmd->cmdBindDescriptorSet(DxvkCmdBuffer::ExecBuffer,
-      VK_PIPELINE_BIND_POINT_GRAPHICS, m_pipelineLayout,
-      set, 0, nullptr);
-
     PushConstants args = { };
     args.srcOffset = srcRect.offset;
     args.srcExtent = srcRect.extent;
     args.dstOffset = dstRect.offset;
+    args.cursorOffset = m_cursorRect.offset;
+    args.cursorExtent = m_cursorRect.extent;
+    args.samplerGamma = m_samplerGamma->getDescriptor().samplerIndex;
+    args.samplerCursor = cursorSampler;
 
-    ctx.cmd->cmdPushConstants(DxvkCmdBuffer::ExecBuffer,
-      m_pipelineLayout, VK_SHADER_STAGE_FRAGMENT_BIT,
-      0, sizeof(args), &args);
+    ctx->cmdBindPipeline(DxvkCmdBuffer::ExecBuffer,
+      VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline);
 
-    ctx.cmd->cmdDraw(3, 1, 0, 0);
+    ctx->bindResources(DxvkCmdBuffer::ExecBuffer,
+      m_blitLayout, descriptors.size(), descriptors.data(),
+      sizeof(args), &args);
+
+    ctx->cmdDraw(3, 1, 0, 0);
+
+    if (unlikely(m_device->debugFlags().test(DxvkDebugFlag::Capture)))
+      ctx->cmdEndDebugUtilsLabel(DxvkCmdBuffer::ExecBuffer);
 
     // Make sure to keep used resources alive
-    ctx.cmd->track(srcView->image(), DxvkAccess::Read);
-    ctx.cmd->track(dstView->image(), DxvkAccess::Write);
+    ctx->track(srcView->image(), DxvkAccess::Read);
+    ctx->track(dstView->image(), DxvkAccess::Write);
 
     if (m_gammaImage)
-      ctx.cmd->track(m_gammaImage, DxvkAccess::Read);
+      ctx->track(m_gammaImage, DxvkAccess::Read);
 
-    ctx.cmd->track(m_samplerGamma);
-    ctx.cmd->track(m_samplerPresent);
+    if (m_hudImage)
+      ctx->track(m_hudImage, DxvkAccess::Read);
+
+    if (m_cursorImage)
+      ctx->track(m_cursorImage, DxvkAccess::Read);
+
+    ctx->track(m_samplerGamma);
+    ctx->track(m_samplerCursorLinear);
+    ctx->track(m_samplerCursorNearest);
+  }
+
+
+  void DxvkSwapchainBlitter::renderHudImage(
+    const Rc<DxvkCommandList>&        ctx,
+          VkExtent3D                  extent) {
+    if (m_hud->empty())
+      return;
+
+    if (!m_hudImage || m_hudImage->info().extent != extent)
+      createHudImage(extent);
+
+    if (unlikely(m_device->debugFlags().test(DxvkDebugFlag::Capture))) {
+      ctx->cmdBeginDebugUtilsLabel(DxvkCmdBuffer::ExecBuffer,
+        vk::makeLabel(0xdcc0f0, "HUD render"));
+    }
+
+    // Reset image
+    VkImageLayout renderLayout = m_hudRtv->getLayout();
+
+    VkImageMemoryBarrier2 barrier = { VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER_2 };
+    barrier.srcStageMask = VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT;
+    barrier.dstAccessMask = VK_ACCESS_2_COLOR_ATTACHMENT_WRITE_BIT;
+    barrier.dstStageMask = VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT;
+    barrier.oldLayout = VK_IMAGE_LAYOUT_UNDEFINED;
+    barrier.newLayout = renderLayout;
+    barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
+    barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
+    barrier.image = m_hudImage->handle();
+    barrier.subresourceRange = m_hudRtv->imageSubresources();
+
+    VkDependencyInfo depInfo = { VK_STRUCTURE_TYPE_DEPENDENCY_INFO };
+    depInfo.imageMemoryBarrierCount = 1;
+    depInfo.pImageMemoryBarriers = &barrier;
+
+    ctx->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
+    m_hudImage->trackInitialization(barrier.subresourceRange);
+
+    // Render actual HUD image
+    VkRenderingAttachmentInfo attachmentInfo = { VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO };
+    attachmentInfo.imageView = m_hudRtv->handle();
+    attachmentInfo.imageLayout = renderLayout;
+    attachmentInfo.loadOp = VK_ATTACHMENT_LOAD_OP_CLEAR;
+    attachmentInfo.storeOp = VK_ATTACHMENT_STORE_OP_STORE;
+
+    VkRenderingInfo renderInfo = { VK_STRUCTURE_TYPE_RENDERING_INFO };
+    renderInfo.renderArea.offset = { 0u, 0u };
+    renderInfo.renderArea.extent = { extent.width, extent.height };
+    renderInfo.layerCount = 1u;
+    renderInfo.colorAttachmentCount = 1;
+    renderInfo.pColorAttachments = &attachmentInfo;
+
+    ctx->cmdBeginRendering(&renderInfo);
+
+    m_hud->render(ctx, m_hudRtv);
+
+    ctx->cmdEndRendering();
+
+    // Make image shader-readable
+    barrier.srcStageMask = VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT;
+    barrier.srcAccessMask = VK_ACCESS_2_COLOR_ATTACHMENT_WRITE_BIT;
+    barrier.dstStageMask = VK_PIPELINE_STAGE_2_FRAGMENT_SHADER_BIT;
+    barrier.dstAccessMask = VK_ACCESS_2_SHADER_READ_BIT;
+    barrier.oldLayout = renderLayout;
+    barrier.newLayout = m_hudImage->info().layout;
+
+    ctx->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
+
+    if (unlikely(m_device->debugFlags().test(DxvkDebugFlag::Capture)))
+      ctx->cmdEndDebugUtilsLabel(DxvkCmdBuffer::ExecBuffer);
+
+    ctx->track(m_hudImage, DxvkAccess::Write);
+  }
+
+
+  void DxvkSwapchainBlitter::createHudImage(
+          VkExtent3D                  extent) {
+    DxvkImageCreateInfo imageInfo = { };
+    imageInfo.type          = VK_IMAGE_TYPE_2D;
+    imageInfo.format        = VK_FORMAT_R8G8B8A8_SRGB;
+    imageInfo.sampleCount   = VK_SAMPLE_COUNT_1_BIT;
+    imageInfo.extent        = extent;
+    imageInfo.mipLevels     = 1;
+    imageInfo.numLayers     = 1;
+    imageInfo.usage         = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT
+                            | VK_IMAGE_USAGE_SAMPLED_BIT;
+    imageInfo.stages        = VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT
+                            | VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT;
+    imageInfo.access        = VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT
+                            | VK_ACCESS_COLOR_ATTACHMENT_READ_BIT
+                            | VK_ACCESS_SHADER_READ_BIT;
+    imageInfo.tiling        = VK_IMAGE_TILING_OPTIMAL;
+    imageInfo.layout        = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
+    imageInfo.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
+    imageInfo.colorSpace    = VK_COLOR_SPACE_SRGB_NONLINEAR_KHR;
+    imageInfo.debugName     = "HUD composition";
+
+    m_hudImage = m_device->createImage(imageInfo, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+
+    DxvkImageViewKey viewInfo = { };
+    viewInfo.viewType       = VK_IMAGE_VIEW_TYPE_2D;
+    viewInfo.format         = imageInfo.format;
+    viewInfo.aspects        = VK_IMAGE_ASPECT_COLOR_BIT;
+    viewInfo.mipIndex       = 0u;
+    viewInfo.mipCount       = 1u;
+    viewInfo.layerIndex     = 0u;
+    viewInfo.layerCount     = 1u;
+
+    viewInfo.usage          = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
+    m_hudRtv = m_hudImage->createView(viewInfo);
+
+    viewInfo.usage          = VK_IMAGE_USAGE_SAMPLED_BIT;
+    m_hudSrv = m_hudImage->createView(viewInfo);
+  }
+
+
+  void DxvkSwapchainBlitter::destroyHudImage() {
+    m_hudImage = nullptr;
+    m_hudRtv = nullptr;
+    m_hudSrv = nullptr;
+  }
+
+
+  void DxvkSwapchainBlitter::renderCursor(
+    const Rc<DxvkCommandList>&        ctx,
+    const Rc<DxvkImageView>&          dstView) {
+    if (!m_cursorRect.extent.width || !m_cursorRect.extent.height)
+      return;
+
+    if (unlikely(m_device->debugFlags().test(DxvkDebugFlag::Capture))) {
+      ctx->cmdBeginDebugUtilsLabel(DxvkCmdBuffer::ExecBuffer,
+        vk::makeLabel(0xdcc0f0, "Software cursor"));
+    }
+
+    VkExtent3D dstExtent = dstView->mipLevelExtent(0u);
+
+    VkViewport viewport = { };
+    viewport.x = 0.0f;
+    viewport.y = 0.0f;
+    viewport.width = float(dstExtent.width);
+    viewport.height = float(dstExtent.height);
+    viewport.minDepth = 0.0f;
+    viewport.maxDepth = 0.0f;
+
+    ctx->cmdSetViewport(1, &viewport);
+
+    VkRect2D scissor = { };
+    scissor.extent.width = dstExtent.width;
+    scissor.extent.height = dstExtent.height;
+
+    ctx->cmdSetScissor(1, &scissor);
+
+    DxvkCursorPipelineKey key = { };
+    key.dstFormat = dstView->info().format;
+    key.dstSpace = dstView->image()->info().colorSpace;
+
+    VkPipeline pipeline = getCursorPipeline(key);
+
+    VkExtent3D cursorExtent = m_cursorImage->info().extent;
+
+    DxvkDescriptorWrite imageDescriptor = { };
+    imageDescriptor.descriptorType = VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE;
+    imageDescriptor.descriptor = m_cursorView->getDescriptor();
+
+    CursorPushConstants args = { };
+    args.dstExtent = { dstExtent.width, dstExtent.height };
+    args.cursorOffset = m_cursorRect.offset;
+    args.cursorExtent = m_cursorRect.extent;
+    args.sampler = m_samplerCursorNearest->getDescriptor().samplerIndex;
+
+    if (m_cursorRect.extent.width != cursorExtent.width
+     || m_cursorRect.extent.height != cursorExtent.height)
+      args.sampler = m_samplerCursorLinear->getDescriptor().samplerIndex;
+
+    ctx->cmdBindPipeline(DxvkCmdBuffer::ExecBuffer,
+      VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline);
+
+    ctx->bindResources(DxvkCmdBuffer::ExecBuffer,
+      m_cursorLayout, 1u, &imageDescriptor, sizeof(args), &args);
+
+    ctx->cmdDraw(4, 1, 0, 0);
+
+    if (unlikely(m_device->debugFlags().test(DxvkDebugFlag::Capture)))
+      ctx->cmdEndDebugUtilsLabel(DxvkCmdBuffer::ExecBuffer);
+
+    ctx->track(m_cursorImage, DxvkAccess::Write);
   }
 
 
   void DxvkSwapchainBlitter::uploadGammaImage(
-    const DxvkContextObjects&         ctx) {
+    const Rc<DxvkCommandList>&        ctx) {
     if (!m_gammaImage || m_gammaImage->info().extent.width != m_gammaCpCount) {
       DxvkImageCreateInfo imageInfo = { };
       imageInfo.type = VK_IMAGE_TYPE_1D;
@@ -397,14 +601,14 @@ namespace dxvk {
 
 
   void DxvkSwapchainBlitter::uploadCursorImage(
-    const DxvkContextObjects&         ctx) {
+    const Rc<DxvkCommandList>&        ctx) {
     uploadTexture(ctx, m_cursorImage, m_cursorBuffer);
     m_cursorBuffer = nullptr;
   }
 
 
   void DxvkSwapchainBlitter::uploadTexture(
-    const DxvkContextObjects&         ctx,
+    const Rc<DxvkCommandList>&        ctx,
     const Rc<DxvkImage>&              image,
     const Rc<DxvkBuffer>&             buffer) {
     VkImageMemoryBarrier2 barrier = { VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER_2 };
@@ -423,10 +627,10 @@ namespace dxvk {
     depInfo.imageMemoryBarrierCount = 1;
     depInfo.pImageMemoryBarriers = &barrier;
 
-    ctx.cmd->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
+    ctx->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
     image->trackInitialization(barrier.subresourceRange);
 
-    DxvkBufferSliceHandle bufferSlice = buffer->getSliceHandle();
+    DxvkResourceBufferInfo bufferSlice = buffer->getSliceInfo();
 
     VkBufferImageCopy2 copyRegion = { VK_STRUCTURE_TYPE_BUFFER_IMAGE_COPY_2 };
     copyRegion.bufferOffset = bufferSlice.offset;
@@ -435,13 +639,13 @@ namespace dxvk {
     copyRegion.imageSubresource.layerCount = 1u;
 
     VkCopyBufferToImageInfo2 copy = { VK_STRUCTURE_TYPE_COPY_BUFFER_TO_IMAGE_INFO_2 };
-    copy.srcBuffer = bufferSlice.handle;
+    copy.srcBuffer = bufferSlice.buffer;
     copy.dstImage = image->handle();
     copy.dstImageLayout = image->pickLayout(VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL);
     copy.regionCount = 1;
     copy.pRegions = &copyRegion;
 
-    ctx.cmd->cmdCopyBufferToImage(DxvkCmdBuffer::ExecBuffer, &copy);
+    ctx->cmdCopyBufferToImage(DxvkCmdBuffer::ExecBuffer, &copy);
 
     barrier.srcStageMask = VK_PIPELINE_STAGE_TRANSFER_BIT;
     barrier.srcAccessMask = VK_ACCESS_TRANSFER_WRITE_BIT;
@@ -450,10 +654,10 @@ namespace dxvk {
     barrier.oldLayout = barrier.newLayout;
     barrier.newLayout = image->info().layout;
 
-    ctx.cmd->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
+    ctx->cmdPipelineBarrier(DxvkCmdBuffer::ExecBuffer, &depInfo);
 
-    ctx.cmd->track(buffer, DxvkAccess::Read);
-    ctx.cmd->track(image, DxvkAccess::Write);
+    ctx->track(buffer, DxvkAccess::Read);
+    ctx->track(image, DxvkAccess::Write);
   }
 
 
@@ -461,14 +665,6 @@ namespace dxvk {
     DxvkSamplerKey samplerInfo = { };
     samplerInfo.setFilter(VK_FILTER_LINEAR, VK_FILTER_LINEAR,
       VK_SAMPLER_MIPMAP_MODE_NEAREST);
-    samplerInfo.setAddressModes(
-      VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER,
-      VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER,
-      VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER);
-    samplerInfo.setUsePixelCoordinates(true);
- 
-    m_samplerPresent = m_device->createSampler(samplerInfo);
-
     samplerInfo.setAddressModes(
       VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE,
       VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE,
@@ -476,113 +672,58 @@ namespace dxvk {
     samplerInfo.setUsePixelCoordinates(false);
  
     m_samplerGamma = m_device->createSampler(samplerInfo);
+
+    samplerInfo.setAddressModes(
+      VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER,
+      VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER,
+      VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER);
+
+    m_samplerCursorLinear = m_device->createSampler(samplerInfo);
+
+    samplerInfo.setFilter(VK_FILTER_NEAREST, VK_FILTER_NEAREST,
+      VK_SAMPLER_MIPMAP_MODE_NEAREST);
+
+    m_samplerCursorNearest = m_device->createSampler(samplerInfo);
   }
 
 
-  void DxvkSwapchainBlitter::createShaders() {
-    createShaderModule(m_shaderVsBlit, VK_SHADER_STAGE_VERTEX_BIT,
-      sizeof(dxvk_present_vert), dxvk_present_vert);
-    createShaderModule(m_shaderFsBlit, VK_SHADER_STAGE_FRAGMENT_BIT,
-      sizeof(dxvk_present_frag_blit), dxvk_present_frag_blit);
-    createShaderModule(m_shaderFsCopy, VK_SHADER_STAGE_FRAGMENT_BIT,
-      sizeof(dxvk_present_frag), dxvk_present_frag);
-    createShaderModule(m_shaderFsMsBlit, VK_SHADER_STAGE_FRAGMENT_BIT,
-      sizeof(dxvk_present_frag_ms_blit), dxvk_present_frag_ms_blit);
-
-    if (m_device->features().amdShaderFragmentMask) {
-      createShaderModule(m_shaderFsMsResolve, VK_SHADER_STAGE_FRAGMENT_BIT,
-        sizeof(dxvk_present_frag_ms_amd), dxvk_present_frag_ms_amd);
-    } else {
-      createShaderModule(m_shaderFsMsResolve, VK_SHADER_STAGE_FRAGMENT_BIT,
-        sizeof(dxvk_present_frag_ms), dxvk_present_frag_ms);
-    }
-  }
-
-
-  void DxvkSwapchainBlitter::createShaderModule(
-          ShaderModule&               shader,
-          VkShaderStageFlagBits       stage,
-          size_t                      size,
-    const uint32_t*                   code) {
-    shader.moduleInfo.codeSize = size;
-    shader.moduleInfo.pCode = code;
-
-    shader.stageInfo.stage = stage;
-    shader.stageInfo.pName = "main";
-
-    if (m_device->features().khrMaintenance5.maintenance5
-     || m_device->features().extGraphicsPipelineLibrary.graphicsPipelineLibrary) {
-      shader.stageInfo.pNext = &shader.moduleInfo;
-      return;
-    }
-
-    auto vk = m_device->vkd();
-
-    VkResult vr = vk->vkCreateShaderModule(vk->device(),
-      &shader.moduleInfo, nullptr, &shader.stageInfo.module);
-
-    if (vr != VK_SUCCESS)
-      throw DxvkError(str::format("Failed to create swap chain blit shader module: ", vr));
-  }
-
-
-  VkDescriptorSetLayout DxvkSwapchainBlitter::createSetLayout() {
-    auto vk = m_device->vkd();
-
-    std::array<VkDescriptorSetLayoutBinding, 3> bindings = {{
-      { 0, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_FRAGMENT_BIT },
-      { 1, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_FRAGMENT_BIT },
-      { 2, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER,         1, VK_SHADER_STAGE_VERTEX_BIT   },
+  const DxvkPipelineLayout* DxvkSwapchainBlitter::createBlitPipelineLayout() {
+    static const std::array<DxvkDescriptorSetLayoutBinding, 4> bindings = {{
+      { VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE, 1, VK_SHADER_STAGE_FRAGMENT_BIT },
+      { VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE, 1, VK_SHADER_STAGE_FRAGMENT_BIT },
+      { VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE, 1, VK_SHADER_STAGE_FRAGMENT_BIT },
+      { VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE, 1, VK_SHADER_STAGE_FRAGMENT_BIT },
     }};
 
-    VkDescriptorSetLayoutCreateInfo info = { VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO };
-    info.bindingCount = bindings.size();
-    info.pBindings = bindings.data();
-
-    VkDescriptorSetLayout layout = VK_NULL_HANDLE;
-    VkResult vr = vk->vkCreateDescriptorSetLayout(vk->device(), &info, nullptr, &layout);
-
-    if (vr != VK_SUCCESS)
-      throw DxvkError(str::format("Failed to create swap chain blit descriptor set layout: ", vr));
-
-    return layout;
+    return m_device->createBuiltInPipelineLayout(DxvkPipelineLayoutFlag::UsesSamplerHeap,
+      VK_SHADER_STAGE_FRAGMENT_BIT, sizeof(PushConstants), bindings.size(), bindings.data());
   }
 
 
-  VkPipelineLayout DxvkSwapchainBlitter::createPipelineLayout() {
-    auto vk = m_device->vkd();
+  const DxvkPipelineLayout* DxvkSwapchainBlitter::createCursorPipelineLayout() {
+    static const std::array<DxvkDescriptorSetLayoutBinding, 1> bindings = {{
+      { VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE, 1, VK_SHADER_STAGE_FRAGMENT_BIT },
+    }};
 
-    VkPushConstantRange pushConst = { };
-    pushConst.size = sizeof(PushConstants);
-    pushConst.stageFlags = VK_SHADER_STAGE_FRAGMENT_BIT;
-
-    VkPipelineLayoutCreateInfo info = { VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO };
-    info.setLayoutCount = 1;
-    info.pSetLayouts = &m_setLayout;
-    info.pushConstantRangeCount = 1;
-    info.pPushConstantRanges = &pushConst;
-
-    VkPipelineLayout layout = VK_NULL_HANDLE;
-    VkResult vr = vk->vkCreatePipelineLayout(vk->device(), &info, nullptr, &layout);
-
-    if (vr != VK_SUCCESS)
-      throw DxvkError(str::format("Failed to create swap chain blit pipeline layout: ", vr));
-
-    return layout;
+    return m_device->createBuiltInPipelineLayout(DxvkPipelineLayoutFlag::UsesSamplerHeap,
+      VK_SHADER_STAGE_VERTEX_BIT | VK_SHADER_STAGE_FRAGMENT_BIT,
+      sizeof(CursorPushConstants), bindings.size(), bindings.data());
   }
 
 
-  VkPipeline DxvkSwapchainBlitter::createPipeline(
-    const DxvkSwapchainPipelineKey& key) {
+  VkPipeline DxvkSwapchainBlitter::createBlitPipeline(
+    const DxvkSwapchainPipelineKey&   key) {
     auto vk = m_device->vkd();
 
-    static const std::array<VkSpecializationMapEntry, 6> specMap = {{
-      { 0, offsetof(SpecConstants, sampleCount),  sizeof(VkSampleCountFlagBits) },
-      { 1, offsetof(SpecConstants, gammaBound),   sizeof(VkBool32) },
-      { 2, offsetof(SpecConstants, srcSpace),     sizeof(VkColorSpaceKHR) },
-      { 3, offsetof(SpecConstants, srcIsSrgb),    sizeof(VkBool32) },
-      { 4, offsetof(SpecConstants, dstSpace),     sizeof(VkColorSpaceKHR) },
-      { 5, offsetof(SpecConstants, dstIsSrgb),    sizeof(VkBool32) },
+    static const std::array<VkSpecializationMapEntry, 8> specMap = {{
+      { 0, offsetof(SpecConstants, sampleCount),    sizeof(VkSampleCountFlagBits) },
+      { 1, offsetof(SpecConstants, gammaBound),     sizeof(VkBool32) },
+      { 2, offsetof(SpecConstants, srcSpace),       sizeof(VkColorSpaceKHR) },
+      { 3, offsetof(SpecConstants, srcIsSrgb),      sizeof(VkBool32) },
+      { 4, offsetof(SpecConstants, dstSpace),       sizeof(VkColorSpaceKHR) },
+      { 5, offsetof(SpecConstants, dstIsSrgb),      sizeof(VkBool32) },
+      { 6, offsetof(SpecConstants, compositeHud),   sizeof(VkBool32) },
+      { 7, offsetof(SpecConstants, compositeCursor),sizeof(VkBool32) },
     }};
 
     SpecConstants specConstants = { };
@@ -592,10 +733,13 @@ namespace dxvk {
     specConstants.srcIsSrgb = key.srcIsSrgb;
     specConstants.dstSpace = key.dstSpace;
     specConstants.dstIsSrgb = lookupFormatInfo(key.dstFormat)->flags.test(DxvkFormatFlag::ColorSpaceSrgb);
+    specConstants.compositeCursor = key.compositeCursor;
+    specConstants.compositeHud = key.compositeHud;
 
     // Avoid redundant color space conversions if color spaces
     // and images properties match and we don't do a resolve
-    if (specConstants.srcSpace == specConstants.dstSpace && key.srcSamples == VK_SAMPLE_COUNT_1_BIT) {
+    if (key.srcSpace == key.dstSpace && key.srcSamples == VK_SAMPLE_COUNT_1_BIT
+     && !key.compositeCursor && !key.compositeHud) {
       specConstants.srcSpace = VK_COLOR_SPACE_PASS_THROUGH_EXT;
       specConstants.dstSpace = VK_COLOR_SPACE_PASS_THROUGH_EXT;
     }
@@ -606,101 +750,109 @@ namespace dxvk {
     specInfo.dataSize = sizeof(specConstants);
     specInfo.pData = &specConstants;
 
-    std::array<VkPipelineShaderStageCreateInfo, 2> blitStages = { };
-    blitStages[0] = m_shaderVsBlit.stageInfo;
+    util::DxvkBuiltInGraphicsState state = { };
+    state.vs = util::DxvkBuiltInShaderStage(dxvk_present_vert, nullptr);
 
-    if (key.srcSamples == VK_SAMPLE_COUNT_1_BIT)
-      blitStages[1] = key.needsBlit ? m_shaderFsBlit.stageInfo : m_shaderFsCopy.stageInfo;
-    else
-      blitStages[1] = key.needsBlit ? m_shaderFsMsBlit.stageInfo : m_shaderFsMsResolve.stageInfo;
-
-    blitStages[1].pSpecializationInfo = &specInfo;
-
-    VkPipelineRenderingCreateInfo rtInfo = { VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO };
-    rtInfo.colorAttachmentCount = 1;
-    rtInfo.pColorAttachmentFormats = &key.dstFormat;
-
-    VkPipelineVertexInputStateCreateInfo viState = { VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO };
-
-    VkPipelineInputAssemblyStateCreateInfo iaState = { VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO };
-    iaState.topology = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST;
-
-    VkPipelineViewportStateCreateInfo vpState = { VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO };
-
-    VkPipelineRasterizationStateCreateInfo rsState = { VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO };
-    rsState.cullMode = VK_CULL_MODE_NONE;
-    rsState.polygonMode = VK_POLYGON_MODE_FILL;
-    rsState.frontFace = VK_FRONT_FACE_COUNTER_CLOCKWISE;
-    rsState.lineWidth = 1.0f;
-
-    constexpr uint32_t sampleMask = 0x1;
-
-    VkPipelineMultisampleStateCreateInfo msState = { VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO };
-    msState.rasterizationSamples = VK_SAMPLE_COUNT_1_BIT;
-    msState.pSampleMask = &sampleMask;
-
-    VkPipelineColorBlendAttachmentState cbAttachment = { };
-    cbAttachment.colorWriteMask =
-      VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT |
-      VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
-
-    if (key.needsBlending) {
-      cbAttachment.blendEnable = VK_TRUE;
-      cbAttachment.colorBlendOp = VK_BLEND_OP_ADD;
-      cbAttachment.srcColorBlendFactor = VK_BLEND_FACTOR_SRC_ALPHA;
-      cbAttachment.dstColorBlendFactor = VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA;
-      cbAttachment.alphaBlendOp = VK_BLEND_OP_ADD;
-      cbAttachment.srcAlphaBlendFactor = VK_BLEND_FACTOR_ONE;
-      cbAttachment.dstAlphaBlendFactor = VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA;
+    if (key.srcSamples == VK_SAMPLE_COUNT_1_BIT) {
+      state.fs = key.needsBlit
+        ? util::DxvkBuiltInShaderStage(dxvk_present_frag_blit, &specInfo)
+        : util::DxvkBuiltInShaderStage(dxvk_present_frag, &specInfo);
+    } else {
+      state.fs = key.needsBlit
+        ? util::DxvkBuiltInShaderStage(dxvk_present_frag_ms_blit, &specInfo)
+        : util::DxvkBuiltInShaderStage(dxvk_present_frag_ms, &specInfo);
     }
 
-    VkPipelineColorBlendStateCreateInfo cbState = { VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO };
-    cbState.attachmentCount = 1;
-    cbState.pAttachments = &cbAttachment;
-
-    static const std::array<VkDynamicState, 2> dynStates = {
-      VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT,
-      VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT,
-    };
-
-    VkPipelineDynamicStateCreateInfo dynState = { VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO };
-    dynState.dynamicStateCount = dynStates.size();
-    dynState.pDynamicStates = dynStates.data();
-
-    VkGraphicsPipelineCreateInfo blitInfo = { VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO, &rtInfo };
-    blitInfo.stageCount = blitStages.size();
-    blitInfo.pStages = blitStages.data();
-    blitInfo.pVertexInputState = &viState;
-    blitInfo.pInputAssemblyState = &iaState;
-    blitInfo.pViewportState = &vpState;
-    blitInfo.pRasterizationState = &rsState;
-    blitInfo.pMultisampleState = &msState;
-    blitInfo.pColorBlendState = &cbState;
-    blitInfo.pDynamicState = &dynState;
-    blitInfo.layout = m_pipelineLayout;
-    blitInfo.basePipelineIndex = -1;
-
-    VkPipeline pipeline = VK_NULL_HANDLE;
-    VkResult vr = vk->vkCreateGraphicsPipelines(vk->device(), VK_NULL_HANDLE,
-      1, &blitInfo, nullptr, &pipeline);
-
-    if (vr != VK_SUCCESS)
-      throw DxvkError(str::format("Failed to create swap chain blit pipeline: ", vr));
-
-    return pipeline;
+    state.colorFormat = key.dstFormat;
+    return m_device->createBuiltInGraphicsPipeline(m_blitLayout, state);
   }
 
 
-  VkPipeline DxvkSwapchainBlitter::getPipeline(
-    const DxvkSwapchainPipelineKey& key) {
+  VkPipeline DxvkSwapchainBlitter::getBlitPipeline(
+    const DxvkSwapchainPipelineKey&   key) {
     auto entry = m_pipelines.find(key);
 
     if (entry != m_pipelines.end())
       return entry->second;
 
-    VkPipeline pipeline = createPipeline(key);
+    VkPipeline pipeline = createBlitPipeline(key);
     m_pipelines.insert({ key, pipeline });
     return pipeline;
   }
 
-}
\ No newline at end of file
+
+  VkPipeline DxvkSwapchainBlitter::createCursorPipeline(
+    const DxvkCursorPipelineKey&      key) {
+    auto vk = m_device->vkd();
+
+    static const std::array<VkSpecializationMapEntry, 2> specMap = {{
+      { 0, offsetof(CursorSpecConstants, dstSpace),  sizeof(VkColorSpaceKHR) },
+      { 1, offsetof(CursorSpecConstants, dstIsSrgb), sizeof(VkBool32) },
+    }};
+
+    CursorSpecConstants specConstants = { };
+    specConstants.dstSpace = key.dstSpace;
+    specConstants.dstIsSrgb = lookupFormatInfo(key.dstFormat)->flags.test(DxvkFormatFlag::ColorSpaceSrgb);
+
+    VkSpecializationInfo specInfo = { };
+    specInfo.mapEntryCount = specMap.size();
+    specInfo.pMapEntries = specMap.data();
+    specInfo.dataSize = sizeof(specConstants);
+    specInfo.pData = &specConstants;
+
+    VkPipelineInputAssemblyStateCreateInfo iaState = { VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO };
+    iaState.topology = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP;
+
+    VkPipelineColorBlendAttachmentState cbAttachment = { };
+    cbAttachment.blendEnable = VK_TRUE;
+    cbAttachment.srcColorBlendFactor = VK_BLEND_FACTOR_ONE;
+    cbAttachment.dstColorBlendFactor = VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA;
+    cbAttachment.colorBlendOp = VK_BLEND_OP_ADD;
+    cbAttachment.srcAlphaBlendFactor = VK_BLEND_FACTOR_ONE;
+    cbAttachment.dstAlphaBlendFactor = VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA;
+    cbAttachment.alphaBlendOp = VK_BLEND_OP_ADD;
+    cbAttachment.colorWriteMask =
+      VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT |
+      VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
+
+    util::DxvkBuiltInGraphicsState state = { };
+    state.vs = util::DxvkBuiltInShaderStage(dxvk_cursor_vert, nullptr);
+    state.fs = util::DxvkBuiltInShaderStage(dxvk_cursor_frag, &specInfo);
+    state.colorFormat = key.dstFormat;
+    state.iaState = &iaState;
+    state.cbAttachment = &cbAttachment;
+
+    return m_device->createBuiltInGraphicsPipeline(m_cursorLayout, state);
+  }
+
+
+  VkPipeline DxvkSwapchainBlitter::getCursorPipeline(
+    const DxvkCursorPipelineKey&      key) {
+    auto entry = m_cursorPipelines.find(key);
+
+    if (entry != m_cursorPipelines.end())
+      return entry->second;
+
+    VkPipeline pipeline = createCursorPipeline(key);
+    m_cursorPipelines.insert({ key, pipeline });
+    return pipeline;
+  }
+
+
+  bool DxvkSwapchainBlitter::needsComposition(
+    const Rc<DxvkImageView>&          dstView) {
+    VkColorSpaceKHR colorSpace = dstView->image()->info().colorSpace;
+
+    switch (colorSpace) {
+      case VK_COLOR_SPACE_SRGB_NONLINEAR_KHR:
+        return !dstView->formatInfo()->flags.test(DxvkFormatFlag::ColorSpaceSrgb);
+
+      case VK_COLOR_SPACE_EXTENDED_SRGB_LINEAR_EXT:
+        return false;
+
+      default:
+        return true;
+    }
+  }
+
+}
