@@ -1,5 +1,7 @@
 #pragma once
 
+#include <mutex>
+
 #include "dxvk_buffer.h"
 #include "dxvk_image.h"
 #include "dxvk_sampler.h"
@@ -25,36 +27,37 @@ namespace dxvk {
     /**
      * \brief Dummy buffer handle
      * 
-     * Returns a handle to a buffer filled
-     * with zeroes. Use for unbound vertex
-     * and index buffers.
+     * Returns a handle to a buffer filled with zeroes.
+     * Use for unbound transform feedback buffers only.
      * \returns Dummy buffer handle
      */
-    VkBuffer bufferHandle() const {
-      return m_buffer->getSliceHandle().handle;
-    }
-    
+    DxvkResourceBufferInfo bufferInfo();
+
     /**
-     * \brief Dummy sampler descriptor
+     * \brief Dummy sampler object
      * 
      * Points to a sampler which was created with
      * reasonable default values. Client APIs may
      * still require different behaviour.
-     * \returns Dummy sampler descriptor
+     * \returns Dummy sampler
      */
-    VkSampler samplerHandle() const {
-      return m_sampler->handle();
-    }
-    
+    DxvkSamplerDescriptor samplerInfo();
+
   private:
     
-    Rc<DxvkSampler> m_sampler;
-    Rc<DxvkBuffer>  m_buffer;
+    DxvkDevice*             m_device;
+
+    std::atomic<bool>       m_bufferCreated = { false };
+    std::atomic<bool>       m_samplerCreated = { false };
+
+    dxvk::mutex             m_mutex;
+    Rc<DxvkSampler>         m_sampler;
+    Rc<DxvkBuffer>          m_buffer;
     
-    Rc<DxvkSampler> createSampler(DxvkDevice* dev);
+    Rc<DxvkSampler> createSampler();
     
-    Rc<DxvkBuffer> createBuffer(DxvkDevice* dev);
+    Rc<DxvkBuffer> createBuffer();
     
   };
   
-}
\ No newline at end of file
+}
