@@ -4,6 +4,15 @@
 
 namespace dxvk::util {
 
+  /**
+   * \brief Debug utils label type
+   */
+  enum class DxvkDebugLabelType : uint32_t {
+    External,               ///< App-provided scope
+    InternalRenderPass,     ///< Internal render pass markers
+    InternalBarrierControl, ///< Barrier control markers
+  };
+
   /**
    * \brief Debug label wrapper
    *
@@ -14,14 +23,16 @@ namespace dxvk::util {
 
   public:
 
-    DxvkDebugLabel() = default;
-
-    DxvkDebugLabel(const VkDebugUtilsLabelEXT& label)
-    : m_text(label.pLabelName ? label.pLabelName : "") {
+    DxvkDebugLabel(const VkDebugUtilsLabelEXT& label, DxvkDebugLabelType type)
+    : m_text(label.pLabelName ? label.pLabelName : ""), m_type(type) {
       for (uint32_t i = 0; i < m_color.size(); i++)
         m_color[i] = label.color[i];
     }
 
+    DxvkDebugLabelType type() const {
+      return m_type;
+    }
+
     VkDebugUtilsLabelEXT get() const {
       VkDebugUtilsLabelEXT label = { VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT };
       label.pLabelName = m_text.c_str();
@@ -34,10 +45,70 @@ namespace dxvk::util {
 
     std::string           m_text;
     std::array<float, 4>  m_color = { };
+    DxvkDebugLabelType    m_type;
 
   };
 
 
+  /**
+   * \brief Built-in shader stage
+   *
+   * Stores pointer to shader code and code size.
+   */
+  struct DxvkBuiltInShaderStage {
+    DxvkBuiltInShaderStage() = default;
+
+    template<size_t N>
+    DxvkBuiltInShaderStage(const uint32_t (&dwords)[N], const VkSpecializationInfo* s)
+    : size(N * sizeof(uint32_t)), code(&dwords[0]), spec(s) { }
+
+    size_t                      size = 0u;
+    const uint32_t*             code = nullptr;
+    const VkSpecializationInfo* spec = nullptr;
+  };
+
+
+  /**
+   * \brief Built-in graphics pipeline state
+   *
+   * For any state not explicitly specified, sane
+   * defaults will be chosen as necessary.
+   */
+  struct DxvkBuiltInGraphicsState {
+    /** Vertex shader. Must be defined. */
+    DxvkBuiltInShaderStage vs;
+    /** Geometry shader. */
+    DxvkBuiltInShaderStage gs;
+    /** Fragment shader. */
+    DxvkBuiltInShaderStage fs;
+    /** Color attachment format. */
+    VkFormat colorFormat = VK_FORMAT_UNDEFINED;
+    /** Depth-stencil attachment format */
+    VkFormat depthFormat = VK_FORMAT_UNDEFINED;
+    /** Sample count */
+    VkSampleCountFlagBits sampleCount = VK_SAMPLE_COUNT_1_BIT;
+    /** Vertex input state. If null, a default state containing
+     *  no vertex attributes or bindings will be used. */
+    const VkPipelineVertexInputStateCreateInfo* viState = nullptr;
+    /** Primitive topology state. If null, this will use a
+     *  triangle list without primitive restart. */
+    const VkPipelineInputAssemblyStateCreateInfo* iaState = nullptr;
+    /** Rasterization state. If null, this will use a default
+     *  state without any back-face culling. */
+    const VkPipelineRasterizationStateCreateInfo* rsState = nullptr;
+    /** Depth-stencil state. If null, no depth or stencil tests
+     *  will be performed by the pipeline. */
+    const VkPipelineDepthStencilStateCreateInfo* dsState = nullptr;
+    /** Blend state for the color attachment. If null, blending
+     *  will be disabled and all color components are written. */
+    const VkPipelineColorBlendAttachmentState* cbAttachment = nullptr;
+    /** Additional dynamic states. These will be added to the
+     *  default set of viewport and scissor states. */
+    uint32_t dynamicStateCount = 0u;
+    const VkDynamicState* dynamicStates = nullptr;
+  };
+
+
   /**
    * \brief Gets pipeline stage flags for shader stages
    * 
