@@ -42,7 +42,7 @@ namespace dxvk::hud {
      * \returns Base offset for next item
      */
     virtual HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -77,11 +77,19 @@ namespace dxvk::hud {
      * \returns Base offset for next item
      */
     void render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer);
 
+    /**
+     * \brief Checks whether the item set is empty
+     * \returns \c true if there are no items
+     */
+    bool empty() const {
+      return m_items.empty();
+    }
+
     /**
      * \brief Creates a HUD item if enabled
      *
@@ -91,7 +99,7 @@ namespace dxvk::hud {
      * \param [in] args Constructor arguments
      */
     template<typename T, typename... Args>
-    void add(const char* name, int32_t at, Args... args) {
+    Rc<T> add(const char* name, int32_t at, Args... args) {
       bool enable = m_enableFull;
 
       if (!enable) {
@@ -102,10 +110,14 @@ namespace dxvk::hud {
       if (at < 0 || at > int32_t(m_items.size()))
         at = m_items.size();
 
+      Rc<T> item;
+
       if (enable) {
-        m_items.insert(m_items.begin() + at,
-          new T(std::forward<Args>(args)...));
+        item = new T(std::forward<Args>(args)...);
+        m_items.insert(m_items.begin() + at, item);
       }
+
+      return item;
     }
 
     template<typename T>
@@ -139,7 +151,7 @@ namespace dxvk::hud {
   public:
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -160,7 +172,7 @@ namespace dxvk::hud {
     ~HudClientApiItem();
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -168,7 +180,8 @@ namespace dxvk::hud {
 
   private:
 
-    std::string m_api;
+    sync::Spinlock  m_mutex;
+    std::string     m_api;
 
   };
 
@@ -185,7 +198,7 @@ namespace dxvk::hud {
     ~HudDeviceInfoItem();
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -214,7 +227,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -246,7 +259,7 @@ namespace dxvk::hud {
     ~HudFrameTimeItem();
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -285,26 +298,22 @@ namespace dxvk::hud {
 
     Rc<DxvkDevice>            m_device;
     Rc<DxvkBuffer>            m_gpuBuffer;
-    Rc<DxvkBufferView>        m_textView;
+    Rc<DxvkBufferView>        m_textWrView;
+    Rc<DxvkBufferView>        m_textRdView;
     Rc<DxvkGpuQuery>          m_query;
 
-    VkDescriptorSetLayout     m_computeSetLayout = VK_NULL_HANDLE;
-    VkPipelineLayout          m_computePipelineLayout = VK_NULL_HANDLE;
+    const DxvkPipelineLayout* m_computePipelineLayout = nullptr;
+    const DxvkPipelineLayout* m_gfxPipelineLayout     = nullptr;
+
     VkPipeline                m_computePipeline = VK_NULL_HANDLE;
 
-    HudShaderModule           m_vs;
-    HudShaderModule           m_fs;
-
-    VkDescriptorSetLayout     m_gfxSetLayout = VK_NULL_HANDLE;
-    VkPipelineLayout          m_gfxPipelineLayout = VK_NULL_HANDLE;
-
     std::unordered_map<HudPipelineKey,
       VkPipeline, DxvkHash, DxvkEq> m_gfxPipelines;
 
     uint32_t                  m_nextDataPoint = 0u;
 
     void processFrameTimes(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
             HudRenderer&        renderer,
             uint32_t            dataPoint,
@@ -312,7 +321,7 @@ namespace dxvk::hud {
             HudPos              maxPos);
 
     void drawFrameTimeGraph(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
             HudRenderer&        renderer,
             uint32_t            dataPoint,
@@ -320,14 +329,12 @@ namespace dxvk::hud {
             HudPos              graphSize);
 
     void createResources(
-      const DxvkContextObjects& ctx);
+      const Rc<DxvkCommandList>&ctx);
 
     void createComputePipeline(
           HudRenderer&          renderer);
 
-    VkDescriptorSetLayout createDescriptorSetLayout();
-
-    VkPipelineLayout createPipelineLayout();
+    const DxvkPipelineLayout* createPipelineLayout();
 
     VkPipeline getPipeline(
             HudRenderer&        renderer,
@@ -356,7 +363,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -397,7 +404,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -409,10 +416,11 @@ namespace dxvk::hud {
 
     DxvkStatCounters  m_prevCounters;
 
-    uint64_t          m_gpCount = 0;
-    uint64_t          m_cpCount = 0;
-    uint64_t          m_rpCount = 0;
-    uint64_t          m_pbCount = 0;
+    uint64_t          m_drawCallCount   = 0;
+    uint64_t          m_drawCount       = 0;
+    uint64_t          m_dispatchCount   = 0;
+    uint64_t          m_renderPassCount = 0;
+    uint64_t          m_barrierCount    = 0;
 
     dxvk::high_resolution_clock::time_point m_lastUpdate
       = dxvk::high_resolution_clock::now();
@@ -434,7 +442,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -455,7 +463,7 @@ namespace dxvk::hud {
    * \brief HUD item to display descriptor stats
    */
   class HudDescriptorStatsItem : public HudItem {
-
+    constexpr static int64_t UpdateInterval = 500'000;
   public:
 
     HudDescriptorStatsItem(const Rc<DxvkDevice>& device);
@@ -465,7 +473,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -478,6 +486,17 @@ namespace dxvk::hud {
     uint64_t m_descriptorPoolCount = 0;
     uint64_t m_descriptorSetCount  = 0;
 
+    uint64_t m_descriptorHeapCount = 0;
+    uint64_t m_descriptorHeapAlloc = 0;
+    uint64_t m_descriptorHeapUsed  = 0;
+    uint64_t m_descriptorHeapMax   = 0;
+    uint64_t m_descriptorHeapPrev  = 0;
+
+    uint64_t m_copyThreadBusyTicks = 0;
+    uint32_t m_copyThreadLoad      = 0u;
+
+    high_resolution_clock::time_point m_lastUpdate = { };
+
   };
 
 
@@ -495,7 +514,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -526,7 +545,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -560,14 +579,7 @@ namespace dxvk::hud {
     Rc<DxvkBuffer>            m_dataBuffer;
     std::vector<DrawInfo>     m_drawInfos;
 
-    HudShaderModule           m_vsBackground;
-    HudShaderModule           m_fsBackground;
-
-    HudShaderModule           m_vsVisualize;
-    HudShaderModule           m_fsVisualize;
-
-    VkDescriptorSetLayout     m_setLayout = VK_NULL_HANDLE;
-    VkPipelineLayout          m_pipelineLayout = VK_NULL_HANDLE;
+    const DxvkPipelineLayout* m_pipelineLayout = nullptr;
 
     std::unordered_map<HudPipelineKey,
       PipelinePair, DxvkHash, DxvkEq> m_pipelines;
@@ -579,19 +591,17 @@ namespace dxvk::hud {
       const DxvkMemoryChunkStats& chunk);
 
     void flushDraws(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer);
 
     void updateDataBuffer(
-      const DxvkContextObjects& ctx,
-            VkDescriptorBufferInfo& drawDescriptor,
-            VkDescriptorBufferInfo& dataDescriptor);
+      const Rc<DxvkCommandList>&ctx,
+            DxvkResourceBufferInfo& drawDescriptor,
+            DxvkResourceBufferInfo& dataDescriptor);
 
-    VkDescriptorSetLayout createSetLayout();
-
-    VkPipelineLayout createPipelineLayout();
+    const DxvkPipelineLayout* createPipelineLayout();
 
     PipelinePair createPipeline(
             HudRenderer&        renderer,
@@ -618,7 +628,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -631,14 +641,18 @@ namespace dxvk::hud {
     uint64_t m_prevCsSyncCount  = 0;
     uint64_t m_prevCsSyncTicks  = 0;
     uint64_t m_prevCsChunks     = 0;
+    uint64_t m_prevCsIdleTicks = 0;
 
     uint64_t m_maxCsSyncCount   = 0;
     uint64_t m_maxCsSyncTicks   = 0;
 
+    uint64_t m_diffCsIdleTicks = 0;
+
     uint64_t m_updateCount      = 0;
 
     std::string m_csSyncString;
     std::string m_csChunkString;
+    std::string m_csLoadString;
 
     dxvk::high_resolution_clock::time_point m_lastUpdate
       = dxvk::high_resolution_clock::now();
@@ -660,7 +674,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -695,7 +709,7 @@ namespace dxvk::hud {
     void update(dxvk::high_resolution_clock::time_point time);
 
     HudPos render(
-      const DxvkContextObjects& ctx,
+      const Rc<DxvkCommandList>&ctx,
       const HudPipelineKey&     key,
       const HudOptions&         options,
             HudRenderer&        renderer,
@@ -719,4 +733,46 @@ namespace dxvk::hud {
 
   };
 
-}
\ No newline at end of file
+
+  /**
+   * \brief Frame latency item
+   */
+  class HudLatencyItem : public HudItem {
+    constexpr static int64_t UpdateInterval = 500'000;
+
+    constexpr static uint32_t MaxInvalidUpdates = 20u;
+  public:
+
+    HudLatencyItem();
+
+    ~HudLatencyItem();
+
+    void accumulateStats(const DxvkLatencyStats& stats);
+
+    void update(dxvk::high_resolution_clock::time_point time);
+
+    HudPos render(
+      const Rc<DxvkCommandList>&ctx,
+      const HudPipelineKey&     key,
+      const HudOptions&         options,
+            HudRenderer&        renderer,
+            HudPos              position);
+
+  private:
+
+    sync::Spinlock      m_mutex;
+
+    DxvkLatencyStats    m_accumStats = { };
+    uint32_t            m_accumFrames = 0u;
+
+    uint32_t            m_invalidUpdates = MaxInvalidUpdates;
+
+    std::string         m_latencyString;
+    std::string         m_sleepString;
+
+    dxvk::high_resolution_clock::time_point m_lastUpdate
+      = dxvk::high_resolution_clock::now();
+
+  };
+
+}
