@@ -8,12 +8,13 @@
   #endif
 #elif defined(__aarch64__) || defined(_M_ARM64) || defined(_M_ARM64EC)
   #define DXVK_ARCH_ARM64
-#else
-#error "Unknown CPU Architecture"
 #endif
 
 #ifdef DXVK_ARCH_X86
   #ifndef _MSC_VER
+    #if defined(_WIN32) && (defined(__AVX__) || defined(__AVX2__))
+      #error "AVX-enabled builds not supported due to stack alignment issues."
+    #endif
     #if defined(__WINE__) && defined(__clang__)
       #pragma push_macro("_WIN32")
       #undef _WIN32
@@ -54,21 +55,19 @@ namespace dxvk::bit {
     return (value >> fst) & ~(~T(0) << (lst - fst + 1));
   }
 
-  inline uint32_t popcntStep(uint32_t n, uint32_t mask, uint32_t shift) {
-    return (n & mask) + ((n & ~mask) >> shift);
+  template<typename T>
+  T popcnt(T n) {
+    n -= ((n >> 1u) & T(0x5555555555555555ull));
+    n = (n & T(0x3333333333333333ull)) + ((n >> 2u) & T(0x3333333333333333ull));
+    n = (n + (n >> 4u)) & T(0x0f0f0f0f0f0f0f0full);
+    n *= T(0x0101010101010101ull);
+    return n >> (8u * (sizeof(T) - 1u));
   }
-  
-  inline uint32_t popcnt(uint32_t n) {
-    n = popcntStep(n, 0x55555555, 1);
-    n = popcntStep(n, 0x33333333, 2);
-    n = popcntStep(n, 0x0F0F0F0F, 4);
-    n = popcntStep(n, 0x00FF00FF, 8);
-    n = popcntStep(n, 0x0000FFFF, 16);
-    return n;
-  }
-  
+
   inline uint32_t tzcnt(uint32_t n) {
     #if defined(_MSC_VER) && !defined(__clang__)
+    if(n == 0)
+      return 32;
     return _tzcnt_u32(n);
     #elif defined(__BMI__)
     return __tzcnt_u32(n);
@@ -105,6 +104,8 @@ namespace dxvk::bit {
 
   inline uint32_t tzcnt(uint64_t n) {
     #if defined(DXVK_ARCH_X86_64) && defined(_MSC_VER) && !defined(__clang__)
+    if(n == 0)
+      return 64;
     return (uint32_t)_tzcnt_u64(n);
     #elif defined(DXVK_ARCH_X86_64) && defined(__BMI__)
     return __tzcnt_u64(n);
@@ -133,8 +134,40 @@ namespace dxvk::bit {
     #endif
   }
 
+  inline uint32_t bsf(uint32_t n) {
+    #if (defined(__GNUC__) || defined(__clang__)) && !defined(__BMI__) && defined(DXVK_ARCH_X86)
+    uint32_t res;
+    asm ("tzcnt %1,%0"
+    : "=r" (res)
+    : "r" (n)
+    : "cc");
+    return res;
+    #else
+    return tzcnt(n);
+    #endif
+  }
+
+  inline uint32_t bsf(uint64_t n) {
+    #if (defined(__GNUC__) || defined(__clang__)) && !defined(__BMI__) && defined(DXVK_ARCH_X86_64)
+    uint64_t res;
+    asm ("tzcnt %1,%0"
+    : "=r" (res)
+    : "r" (n)
+    : "cc");
+    return res;
+    #else
+    return tzcnt(n);
+    #endif
+  }
+
   inline uint32_t lzcnt(uint32_t n) {
-    #if (defined(_MSC_VER) && !defined(__clang__)) || defined(__LZCNT__)
+    #if defined(_MSC_VER) && !defined(__clang__) && !defined(__LZCNT__)
+    unsigned long bsr;
+    if(n == 0)
+      return 32;
+    _BitScanReverse(&bsr, n);
+    return 31-bsr;
+    #elif (defined(_MSC_VER) && !defined(__clang__)) || defined(__LZCNT__)
     return _lzcnt_u32(n);
     #elif defined(__GNUC__) || defined(__clang__)
     return n != 0 ? __builtin_clz(n) : 32;
@@ -154,7 +187,13 @@ namespace dxvk::bit {
   }
 
   inline uint32_t lzcnt(uint64_t n) {
-    #if defined(DXVK_ARCH_X86_64) && ((defined(_MSC_VER) && !defined(__clang__)) || defined(__LZCNT__))
+    #if defined(_MSC_VER) && !defined(__clang__) && !defined(__LZCNT__) && defined(DXVK_ARCH_X86_64)
+    unsigned long bsr;
+    if(n == 0)
+      return 64;
+    _BitScanReverse64(&bsr, n);
+    return 63-bsr;
+    #elif defined(DXVK_ARCH_X86_64) && ((defined(_MSC_VER) && !defined(__clang__)) && defined(__LZCNT__))
     return _lzcnt_u64(n);
     #elif defined(DXVK_ARCH_X86_64) && (defined(__GNUC__) || defined(__clang__))
     return n != 0 ? __builtin_clzll(n) : 64;
@@ -494,6 +533,7 @@ namespace dxvk::bit {
 
   };
 
+  template<typename T>
   class BitMask {
 
   public:
@@ -501,12 +541,12 @@ namespace dxvk::bit {
     class iterator {
     public:
       using iterator_category = std::input_iterator_tag;
-      using value_type = uint32_t;
-      using difference_type = uint32_t;
-      using pointer = const uint32_t*;
-      using reference = uint32_t;
+      using value_type = T;
+      using difference_type = T;
+      using pointer = const T*;
+      using reference = T;
 
-      explicit iterator(uint32_t flags)
+      explicit iterator(T flags)
         : m_mask(flags) { }
 
       iterator& operator ++ () {
@@ -520,17 +560,8 @@ namespace dxvk::bit {
         return retval;
       }
 
-      uint32_t operator * () const {
-#if (defined(__GNUC__) || defined(__clang__)) && !defined(__BMI__) && defined(DXVK_ARCH_X86)
-        uint32_t res;
-        asm ("tzcnt %1,%0"
-        : "=r" (res)
-        : "r" (m_mask)
-        : "cc");
-        return res;
-#else
-        return tzcnt(m_mask);
-#endif
+      T operator * () const {
+        return bsf(m_mask);
       }
 
       bool operator == (iterator other) const { return m_mask == other.m_mask; }
@@ -538,14 +569,14 @@ namespace dxvk::bit {
 
     private:
 
-      uint32_t m_mask;
+      T m_mask;
 
     };
 
     BitMask()
       : m_mask(0) { }
 
-    BitMask(uint32_t n)
+    explicit BitMask(T n)
       : m_mask(n) { }
 
     iterator begin() {
@@ -558,7 +589,7 @@ namespace dxvk::bit {
 
   private:
 
-    uint32_t m_mask;
+    T m_mask;
 
   };
 
@@ -618,4 +649,71 @@ namespace dxvk::bit {
     return float(n) / float(1u << F);
   }
 
+
+  /**
+   * \brief Inserts one null bit after each bit
+   */
+  inline uint32_t split2(uint32_t c) {
+    c = (c ^ (c << 8u)) & 0x00ff00ffu;
+    c = (c ^ (c << 4u)) & 0x0f0f0f0fu;
+    c = (c ^ (c << 2u)) & 0x33333333u;
+    c = (c ^ (c << 1u)) & 0x55555555u;
+    return c;
+  }
+
+
+  /**
+   * \brief Inserts two null bits after each bit
+   */
+  inline uint64_t split3(uint64_t c) {
+    c = (c | c << 32u) & 0x001f00000000ffffull;
+    c = (c | c << 16u) & 0x001f0000ff0000ffull;
+    c = (c | c <<  8u) & 0x100f00f00f00f00full;
+    c = (c | c <<  4u) & 0x10c30c30c30c30c3ull;
+    c = (c | c <<  2u) & 0x1249249249249249ull;
+    return c;
+  }
+
+
+  /**
+   * \brief Interleaves bits from two integers
+   *
+   * Both numbers must fit into 16 bits.
+   * \param [in] x X coordinate
+   * \param [in] y Y coordinate
+   * \returns Morton code of x and y
+   */
+  inline uint32_t interleave(uint16_t x, uint16_t y) {
+    return split2(x) | (split2(y) << 1u);
+  }
+
+
+  /**
+   * \brief Interleaves bits from three integers
+   *
+   * All three numbers must fit into 16 bits.
+   */
+  inline uint64_t interleave(uint16_t x, uint16_t y, uint16_t z) {
+    return split3(x) | (split3(y) << 1u) | (split3(z) << 2u);
+  }
+
+
+  /**
+   * \brief 48-bit integer storage type
+   */
+  struct uint48_t {
+    explicit uint48_t(uint64_t n)
+    : a(uint16_t(n)), b(uint16_t(n >> 16)), c(uint16_t(n >> 32)) { }
+
+    uint16_t a;
+    uint16_t b;
+    uint16_t c;
+
+    explicit operator uint64_t () const {
+      // GCC generates worse code if we promote to uint64 directly
+      uint32_t lo = uint32_t(a) | (uint32_t(b) << 16);
+      return uint64_t(lo) | (uint64_t(c) << 32);
+    }
+  };
+
 }
